# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.fedb import FEDB
from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from datetime import datetime, timedelta
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.ticker import FuncFormatter
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False
sns.set_style('white', {'font.sans-serif': ['simhei', 'Arial']})
cell_dic = {1: 'A', 2: 'B', 3: 'C', 4: 'D', 5: 'E', 6: 'F', 7: 'G', 8: 'H', 9: 'I', 10: 'J', 11: 'K', 12: 'L', 13: 'M',
            14: 'N', 15: 'O', 16: 'P', 17: 'Q', 18: 'R', 19: 'S', 20: 'T', 21: 'U', 22: 'V', 23: 'W', 24: 'X', 25: 'Y', 26: 'Z'}

def to_percent(temp, position):
    return '%1.0f'%(temp) + '%'

def holding_operation(stock, fund_code, fund_start_date=None):
    all_fund = HBDB().read_fund_info()
    all_fund = all_fund.rename(columns={'jjdm': 'FUND_CODE', 'jjjc': 'FUND_NAME'})
    all_fund = all_fund[all_fund['FUND_CODE'] == fund_code]
    all_fund = all_fund[['FUND_CODE', 'FUND_NAME']].drop_duplicates('FUND_CODE', keep='last')
    fund_name = all_fund['FUND_NAME'].values[0]

    calendar_df, report_df, trade_df, report_trade_df, calendar_trade_df = get_date('19900101', datetime.today().strftime('%Y%m%d'))

    if stock[:2] != '68':
        stock_daily_k = HBDB().read_stock_daily_k_given_code(stock)
    else:
        stock_daily_k = HBDB().read_star_stock_daily_k_given_code(stock)
    stock_daily_k['TRADE_DATE'] = stock_daily_k['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
    stock_daily_k = stock_daily_k.sort_values('TRADE_DATE')
    stock_name = stock_daily_k['SEC_SHORT_NAME'].values[0]

    fund_holding = HBDB().read_fund_holding_given_codes([fund_code])
    fund_holding = fund_holding.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zqdm': 'TICKER_SYMBOL', 'zqmc': 'SEC_SHORT_NAME', 'ccsz': 'HOLDING_MARKET_VALUE', 'ccsl': 'HOLDING_AMOUNT', 'zjbl': 'MV_IN_NA'})
    fund_holding['REPORT_DATE'] = fund_holding['REPORT_DATE'].astype(str)
    fund_start_date = fund_holding['REPORT_DATE'].min() if fund_start_date is None else fund_start_date
    fund_holding_season = fund_holding.copy(deep=True)
    fund_holding_season['MONTH'] = fund_holding_season['REPORT_DATE'].apply(lambda x: x[4:6])
    fund_holding_season = fund_holding_season[fund_holding_season['MONTH'].isin(['03', '09'])]
    fund_holding_min = fund_holding_season[['REPORT_DATE', 'MV_IN_NA']].groupby(['REPORT_DATE']).min().reset_index().rename(columns={'MV_IN_NA': 'MV_IN_NA_MIN'})
    fund_holding_min['MV_IN_NA_MIN'] = fund_holding_min['MV_IN_NA_MIN'].apply(lambda x: '<{0}'.format(round(x, 2)))
    fund_holding = fund_holding[fund_holding['TICKER_SYMBOL'] == stock]
    disp_dates = report_df[report_df['REPORT_DATE'] >= fund_start_date]['REPORT_DATE'].unique().tolist()
    fund_holding = fund_holding.set_index('REPORT_DATE')
    fund_holding = fund_holding.reindex(disp_dates)
    fund_holding.loc[(fund_holding.index.str.slice(4, 6).isin(['06', '12'])) & (fund_holding['MV_IN_NA'].isnull()), 'MV_IN_NA'] = 0.0
    fund_holding = pd.concat([fund_holding, fund_holding_min[['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE')], axis=1)
    fund_holding.loc[(fund_holding.index.str.slice(4, 6).isin(['03', '09'])) & (fund_holding['MV_IN_NA'].isnull()), 'MV_IN_NA'] = fund_holding.loc[(fund_holding.index.str.slice(4, 6).isin(['03', '09'])) & (fund_holding['MV_IN_NA'].isnull()), 'MV_IN_NA_MIN']
    fund_holding = fund_holding.drop('MV_IN_NA_MIN', axis=1)
    fund_holding['MV_IN_NA_STR'] = fund_holding['MV_IN_NA'].astype(str)
    fund_holding = fund_holding.reset_index()
    fund_holding = fund_holding.merge(calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    fund_holding['TRADE_DATE'] = fund_holding['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
    fund_holding = fund_holding.sort_values('TRADE_DATE')
    fund_holding = fund_holding.merge(stock_daily_k[['TRADE_DATE', 'CLOSE_PRICE']], on=['TRADE_DATE'], how='left')
    fund_holding_quancang = fund_holding.loc[fund_holding['REPORT_DATE'].str.slice(4, 6).isin(['06', '12'])]
    fund_holding_quancang['MV_IN_NA'] = fund_holding_quancang['MV_IN_NA'].astype(float)

    stock_daily_k = stock_daily_k[stock_daily_k['TRADE_DATE'] >= datetime.strptime(fund_start_date, '%Y%m%d') - timedelta(30)]
    data = stock_daily_k[['TRADE_DATE', 'CLOSE_PRICE']].merge(fund_holding[['TRADE_DATE', 'MV_IN_NA_STR']], on=['TRADE_DATE'], how='left').merge(fund_holding_quancang[['TRADE_DATE', 'MV_IN_NA']], on=['TRADE_DATE'], how='left')
    data['INDEX'] = range(len(data))
    dates = data[['TRADE_DATE']].sort_values('TRADE_DATE')
    dates['YEAR_MONTH'] = dates['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d')[:6])
    dates = dates.drop_duplicates('YEAR_MONTH', keep='last')
    dates = dates.loc[dates['YEAR_MONTH'].str.slice(4, 6).isin(['03', '06', '09', '12'])]
    data = data.merge(dates[['TRADE_DATE', 'YEAR_MONTH']], on=['TRADE_DATE'], how='left')

    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()
    ax1.plot(data['INDEX'], data['CLOSE_PRICE'], color='#F04950', label='{0}'.format(stock_name))
    ax2.bar(data['INDEX'], data['MV_IN_NA'], width=len(stock_daily_k) / len(fund_holding_quancang) / 10, color='#C94649', label='占净值比（%）')
    ax2.set_ylim([0, 50])
    ax1.scatter(data.dropna(subset=['MV_IN_NA_STR'])['INDEX'], data.dropna(subset=['MV_IN_NA_STR'])['CLOSE_PRICE'], color='#6268A2', label='{0}中该个股持仓占净值比（%）'.format(fund_name))
    for x, y, value in zip(list(data.dropna(subset=['MV_IN_NA_STR'])['INDEX']), list(data.dropna(subset=['MV_IN_NA_STR'])['CLOSE_PRICE']), list(data.dropna(subset=['MV_IN_NA_STR'])['MV_IN_NA_STR'])):
        ax1.text(x, y * 1.05, value, ha='center', va='bottom')
    plt.xticks(ticks=data.dropna(subset=['YEAR_MONTH'])['INDEX'], labels=data.dropna(subset=['YEAR_MONTH'])['YEAR_MONTH'], rotation=90)
    ax1.set_xlabel('')
    ax1.set_ylabel('股价（元）')
    ax2.set_ylabel('占净值比（%）')
    ax1.legend(loc=2)
    ax2.legend(loc=1)
    ax2.yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.tight_layout()
    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/{1}.png'.format(fund_code, stock_name))
    # plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/{0}_{1}.png'.format(fund_name, stock_name))
    return

def get_date(start_date, end_date):
    calendar_df = HBDB().read_cal(start_date, end_date)
    calendar_df = calendar_df.rename(columns={'JYRQ': 'CALENDAR_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    calendar_df['CALENDAR_DATE'] = calendar_df['CALENDAR_DATE'].astype(str)
    calendar_df = calendar_df.sort_values('CALENDAR_DATE')
    calendar_df['IS_OPEN'] = calendar_df['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    calendar_df['YEAR_MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[:6])
    calendar_df['MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:6])
    calendar_df['MONTH_DAY'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:])
    calendar_df = calendar_df[(calendar_df['CALENDAR_DATE'] >= start_date) & (calendar_df['CALENDAR_DATE'] <= end_date)]
    report_df = calendar_df.drop_duplicates('YEAR_MONTH', keep='last').rename(columns={'CALENDAR_DATE': 'REPORT_DATE'})
    report_df = report_df[report_df['MONTH_DAY'].isin(['0331', '0630', '0930', '1231'])]
    report_df = report_df[(report_df['REPORT_DATE'] >= start_date) & (report_df['REPORT_DATE'] <= end_date)]
    trade_df = calendar_df[calendar_df['IS_OPEN'] == 1].rename(columns={'CALENDAR_DATE': 'TRADE_DATE'})
    trade_df = trade_df[(trade_df['TRADE_DATE'] >= start_date) & (trade_df['TRADE_DATE'] <= end_date)]
    report_trade_df = calendar_df[calendar_df['IS_OPEN'] == 1].rename(columns={'CALENDAR_DATE': 'TRADE_DATE'})
    report_trade_df = report_trade_df.sort_values('TRADE_DATE').drop_duplicates('YEAR_MONTH', keep='last')
    report_trade_df = report_trade_df[report_trade_df['MONTH'].isin(['03', '06', '09', '12'])]
    report_trade_df = report_trade_df[(report_trade_df['TRADE_DATE'] >= start_date) & (report_trade_df['TRADE_DATE'] <= end_date)]
    calendar_trade_df = calendar_df[['CALENDAR_DATE']].merge(trade_df[['TRADE_DATE']], left_on=['CALENDAR_DATE'], right_on=['TRADE_DATE'], how='left')
    calendar_trade_df['TRADE_DATE'] = calendar_trade_df['TRADE_DATE'].fillna(method='ffill')
    calendar_trade_df = calendar_trade_df[(calendar_trade_df['TRADE_DATE'] >= start_date) & (calendar_trade_df['TRADE_DATE'] <= end_date)]
    return calendar_df, report_df, trade_df, report_trade_df, calendar_trade_df

class HoldingCompare:
    def __init__(self, target_fund_code, compare_fund_code_list, sw_type=1, begin_date=None):
        self.target_fund_code = target_fund_code
        self.compare_fund_code_list = compare_fund_code_list
        self.sw_type = sw_type
        self.begin_date = begin_date
        self.load()

    def load(self):
        # 基金信息数据
        self.all_fund = HBDB().read_fund_info()
        self.all_fund = self.all_fund.rename(columns={'jjdm': 'FUND_CODE', 'jjjc': 'FUND_NAME'})
        self.all_fund = self.all_fund[self.all_fund['FUND_CODE'].isin([self.target_fund_code] + self.compare_fund_code_list)]
        self.all_fund = self.all_fund[['FUND_CODE', 'FUND_NAME']].drop_duplicates('FUND_CODE', keep='last')
        # 行业信息数据
        self.industry_info = HBDB().read_industry_info()
        self.industry_info = self.industry_info.rename(columns={'flmc': 'INDUSTRY_NAME', 'zsdm': 'INDUSTRY_ID', 'hyhfbz': 'INDUSTRY_VERSION', 'fljb': 'INDUSTRY_TYPE', 'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW'})
        self.industry_info = self.industry_info.dropna(subset=['BEGIN_DATE'])
        self.industry_info['END_DATE'] = self.industry_info['END_DATE'].replace('', np.nan).fillna('20990101')
        self.industry_info['BEGIN_DATE'] = self.industry_info['BEGIN_DATE'].apply(lambda x: datetime.strptime(x, '%Y/%m/%d').strftime('%Y%m%d') if '/' in x else x)
        self.industry_info['END_DATE'] = self.industry_info['END_DATE'].apply(lambda x: datetime.strptime(x, '%Y/%m/%d').strftime('%Y%m%d') if '/' in x else x)
        self.industry_info['BEGIN_DATE'] = self.industry_info['BEGIN_DATE'].astype(int).astype(str)
        self.industry_info['END_DATE'] = self.industry_info['END_DATE'].astype(int).astype(str)
        self.industry_info['INDUSTRY_VERSION'] = self.industry_info['INDUSTRY_VERSION'].astype(int)
        self.industry_info['INDUSTRY_TYPE'] = self.industry_info['INDUSTRY_TYPE'].astype(int)
        self.industry_info['IS_NEW'] = self.industry_info['IS_NEW'].astype(int)
        self.industry_info = self.industry_info[self.industry_info['INDUSTRY_VERSION'] == 3]
        self.industry_info = self.industry_info[self.industry_info['INDUSTRY_TYPE'] == self.sw_type]
        self.industry_info = self.industry_info[self.industry_info['IS_NEW'] == 1]
        self.industry_info = self.industry_info[['INDUSTRY_NAME', 'INDUSTRY_ID', 'BEGIN_DATE', 'END_DATE']]
        self.industry_id_name_dic = self.industry_info[['INDUSTRY_ID', 'INDUSTRY_NAME']].set_index('INDUSTRY_ID')['INDUSTRY_NAME'].to_dict()
        # 个股行业对应数据
        self.stock_industry = HBDB().read_stock_industry()
        self.stock_industry.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_industry.hdf', key='table', mode='w')
        # self.stock_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_industry.hdf', key='table')
        self.stock_industry = self.stock_industry.rename(columns={'zqdm': 'TICKER_SYMBOL', 'flmc': 'INDUSTRY_NAME', 'fldm': 'INDUSTRY_ID', 'fljb': 'INDUSTRY_TYPE', 'hyhfbz': 'INDUSTRY_VERSION', 'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW', 'credt_etl': 'CREDT_ETL'})
        self.stock_industry['INDUSTRY_VERSION'] = self.stock_industry['INDUSTRY_VERSION'].astype(int)
        self.stock_industry['INDUSTRY_TYPE'] = self.stock_industry['INDUSTRY_TYPE'].astype(int)
        self.stock_industry['IS_NEW'] = self.stock_industry['IS_NEW'].astype(int)
        self.stock_industry = self.stock_industry[self.stock_industry['IS_NEW'] == 1]
        self.stock_industry = self.stock_industry.sort_values(['INDUSTRY_VERSION', 'INDUSTRY_TYPE', 'TICKER_SYMBOL', 'CREDT_ETL']).drop_duplicates(['INDUSTRY_VERSION', 'INDUSTRY_TYPE', 'TICKER_SYMBOL'], keep='last')
        self.stock_industry_sw = self.stock_industry[self.stock_industry['INDUSTRY_VERSION'] == 2]
        # 个股持仓数据
        self.all_fund_holding = HBDB().read_fund_holding_given_codes([self.target_fund_code] + self.compare_fund_code_list)
        self.all_fund_holding = self.all_fund_holding.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zqdm': 'TICKER_SYMBOL', 'zqmc': 'SEC_SHORT_NAME', 'ccsz': 'HOLDING_MARKET_VALUE', 'ccsl': 'HOLDING_AMOUNT', 'zjbl': 'MV_IN_NA'})
        self.all_fund_holding = self.all_fund_holding.sort_values(['FUND_CODE', 'REPORT_DATE', 'MV_IN_NA'], ascending=[True, True, False])
        self.all_fund_holding['REPORT_DATE'] = self.all_fund_holding['REPORT_DATE'].astype(str)
        self.all_fund_holding = self.all_fund_holding[self.all_fund_holding['MV_IN_NA'] != 0.0]
        # 基金季报前十大最小权重
        self.all_fund_holding_season = self.all_fund_holding.copy(deep=True)
        self.all_fund_holding_season['MONTH'] = self.all_fund_holding_season['REPORT_DATE'].apply(lambda x: x[4:6])
        self.all_fund_holding_season = self.all_fund_holding_season[self.all_fund_holding_season['MONTH'].isin(['03', '09'])]
        self.all_fund_holding_min = self.all_fund_holding_season[['FUND_CODE', 'REPORT_DATE', 'MV_IN_NA']].groupby(['FUND_CODE', 'REPORT_DATE']).min().reset_index().rename(columns={'MV_IN_NA': 'MV_IN_NA_MIN'})
        self.all_fund_holding_min['MV_IN_NA_MIN'] = self.all_fund_holding_min['MV_IN_NA_MIN'].apply(lambda x: '<{0}'.format(round(x, 2)))
        # 行业持仓数据
        self.all_fund_industry_holding = self.all_fund_holding.merge(self.stock_industry_sw[self.stock_industry_sw['INDUSTRY_TYPE'] == self.sw_type][['TICKER_SYMBOL', 'INDUSTRY_NAME']], on=['TICKER_SYMBOL'], how='left')
        self.all_fund_industry_holding = self.all_fund_industry_holding.dropna(subset=['FUND_CODE', 'REPORT_DATE', 'TICKER_SYMBOL', 'INDUSTRY_NAME', 'MV_IN_NA'])
        self.all_fund_industry_holding = self.all_fund_industry_holding[['FUND_CODE', 'REPORT_DATE', 'INDUSTRY_NAME', 'MV_IN_NA']].groupby(['FUND_CODE', 'REPORT_DATE', 'INDUSTRY_NAME']).sum().reset_index()
        # 日历数据
        self.all_calendar_df, self.all_report_df, self.all_trade_df, self.all_report_trade_df, self.all_calendar_trade_df = get_date('19900101',datetime.today().strftime('%Y%m%d'))
        # 个股行情数据
        stock_daily_k_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_daily_k.hdf'
        if os.path.isfile(stock_daily_k_path):
            existed_stock_daily_k = pd.read_hdf(stock_daily_k_path, key='table')
            max_date = max(existed_stock_daily_k['TRADE_DATE'])
            start_date = max(str(max_date), '20150101')
        else:
            existed_stock_daily_k = pd.DataFrame()
            start_date = '20150101'
        trade_df = self.all_trade_df[(self.all_trade_df['TRADE_DATE'] > start_date) & (self.all_trade_df['TRADE_DATE'] < datetime.today().strftime('%Y%m%d'))]
        data = []
        for idx, td in enumerate(trade_df['TRADE_DATE'].unique().tolist()):
            stock_daily_k_date = HBDB().read_stock_daily_k_given_date(td)
            star_stock_daily_k_date = HBDB().read_star_stock_daily_k_given_date(td)
            stock_daily_k_date = pd.concat([stock_daily_k_date, star_stock_daily_k_date])
            data.append(stock_daily_k_date)
            print('[PreloadStockDailyK][{0}/{1}]'.format(idx, len(trade_df['TRADE_DATE'])))
        self.all_stock_daily_k = pd.concat([existed_stock_daily_k] + data, ignore_index=True)
        self.all_stock_daily_k = self.all_stock_daily_k.sort_values(['TRADE_DATE', 'TICKER_SYMBOL']).drop_duplicates(['TRADE_DATE', 'TICKER_SYMBOL'])
        self.all_stock_daily_k.to_hdf(stock_daily_k_path, key='table', mode='w')
        # self.all_stock_daily_k = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_daily_k.hdf', key='table')
        # 个股名称对应数据
        self.all_stock_name = self.all_stock_daily_k.sort_values(['TICKER_SYMBOL', 'TRADE_DATE'])[['TICKER_SYMBOL', 'SEC_SHORT_NAME']].drop_duplicates('TICKER_SYMBOL', keep='last')
        # 个股估值数据
        stock_valuation_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_valuation.hdf'
        if os.path.isfile(stock_valuation_path):
            existed_stock_valuation = pd.read_hdf(stock_valuation_path, key='table')
            max_date = max(existed_stock_valuation['TRADE_DATE'])
            start_date = max(str(max_date), '20150101')
        else:
            existed_stock_valuation = pd.DataFrame()
            start_date = '20150101'
        trade_df = self.all_trade_df[(self.all_trade_df['TRADE_DATE'] > start_date) & (self.all_trade_df['TRADE_DATE'] < datetime.today().strftime('%Y%m%d'))]
        data = []
        for idx, td in enumerate(trade_df['TRADE_DATE'].unique().tolist()):
            stock_valuation_date = HBDB().read_stock_valuation_given_date(td)
            star_stock_valuation_date = HBDB().read_star_stock_valuation_given_date(td)
            stock_valuation_date = pd.concat([stock_valuation_date, star_stock_valuation_date])
            data.append(stock_valuation_date)
            print('[PreloadStockValuation][{0}/{1}]'.format(idx, len(trade_df['TRADE_DATE'])))
        self.all_stock_valuation = pd.concat([existed_stock_valuation] + data, ignore_index=True)
        self.all_stock_valuation = self.all_stock_valuation.sort_values(['TRADE_DATE', 'TICKER_SYMBOL']).drop_duplicates(['TRADE_DATE', 'TICKER_SYMBOL'])
        self.all_stock_valuation.to_hdf(stock_valuation_path, key='table', mode='w')
        # self.all_stock_valuation = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_valuation.hdf', key='table')
        # 个股财务数据
        stock_finance_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_finance.hdf'
        if os.path.isfile(stock_finance_path):
            existed_stock_finance = pd.read_hdf(stock_finance_path, key='table')
            max_date = max(existed_stock_finance['END_DATE'])
            start_date = max(str(max_date), '20150101')
        else:
            existed_stock_finance = pd.DataFrame()
            start_date = '20150101'
        report_df = self.all_report_df[(self.all_report_df['REPORT_DATE'] > start_date) & (self.all_report_df['REPORT_DATE'] < datetime.today().strftime('%Y%m%d'))]
        data = []
        for idx, td in enumerate(report_df['REPORT_DATE'].unique().tolist()):
            stock_finance_date = HBDB().read_stock_finance_given_date(td)
            star_stock_finance_date = HBDB().read_star_stock_finance_given_date(td)
            stock_finance_date = pd.concat([stock_finance_date, star_stock_finance_date])
            data.append(stock_finance_date)
            print('[PreloadFinance][{0}/{1}]'.format(idx, len(report_df['REPORT_DATE'])))
        self.all_stock_finance = pd.concat([existed_stock_finance] + data, ignore_index=True)
        self.all_stock_finance.to_hdf(stock_finance_path, key='table', mode='w')
        # self.all_stock_finance = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_finance.hdf', key='table')
        # 行业行情数据
        self.all_industry_daily_k = HBDB().read_index_daily_k_given_date_and_indexs('20150101', self.industry_info['INDUSTRY_ID'].unique().tolist())
        self.all_industry_daily_k.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/all_industry_daily_k.hdf', key='table', mode='w')
        # self.all_industry_daily_k = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/all_industry_daily_k.hdf', key='table')
        self.all_industry_daily_k = self.all_industry_daily_k.rename(columns={'zqdm': 'INDUSTRY_ID', 'jyrq': 'TRADE_DATE', 'spjg': 'CLOSE_INDEX'})
        self.all_industry_daily_k['TRADE_DATE'] = self.all_industry_daily_k['TRADE_DATE'].astype(str)
        self.all_industry_daily_k = self.all_industry_daily_k.sort_values(['TRADE_DATE', 'INDUSTRY_ID'])
        self.all_industry_daily_k['INDUSTRY_NAME'] = self.all_industry_daily_k['INDUSTRY_ID'].apply(lambda x: self.industry_id_name_dic[x])
        # 行业估值数据
        self.all_industry_valuation = FEDB().read_industry_valuation(self.sw_type)
        # 行业财务数据
        self.all_industry_finance = FEDB().read_industry_fundamental(self.sw_type)

    def get_stock_holding_compare(self, worksheet_h, worksheet_hc):
        target_start_date = self.all_fund_holding[self.all_fund_holding['FUND_CODE'] == self.target_fund_code]['REPORT_DATE'].min()
        all_fund_holding_cut = self.all_fund_holding[self.all_fund_holding['REPORT_DATE'] >= target_start_date]
        fund_holding_list = []
        for fund_code in [self.target_fund_code] + self.compare_fund_code_list:
            # 个股仓位情况
            fund_holding = all_fund_holding_cut[all_fund_holding_cut['FUND_CODE'] == fund_code]
            fund_holding = fund_holding.pivot(index='TICKER_SYMBOL', columns='REPORT_DATE', values='MV_IN_NA')
            fund_holding_date_columns = sorted(list(fund_holding.columns))
            rank_list = list(fund_holding[fund_holding_date_columns[-1]].sort_values(ascending=False).index)
            fund_holding = fund_holding.reset_index()
            fund_holding['TICKER_SYMBOL'] = fund_holding['TICKER_SYMBOL'].astype('category')
            fund_holding['TICKER_SYMBOL'].cat.reorder_categories(rank_list, inplace=True)
            fund_holding = fund_holding.sort_values('TICKER_SYMBOL')
            fund_holding = fund_holding.merge(self.all_stock_name[['TICKER_SYMBOL', 'SEC_SHORT_NAME']], on=['TICKER_SYMBOL'], how='left')
            fund_holding = fund_holding.merge(self.stock_industry_sw[self.stock_industry_sw['INDUSTRY_TYPE'] == 1][['TICKER_SYMBOL', 'INDUSTRY_NAME']].rename(columns={'INDUSTRY_NAME': 'INDUSTRY_NAME_SW1'}), on=['TICKER_SYMBOL'], how='left')
            fund_holding = fund_holding.merge(self.stock_industry_sw[self.stock_industry_sw['INDUSTRY_TYPE'] == 2][['TICKER_SYMBOL', 'INDUSTRY_NAME']].rename(columns={'INDUSTRY_NAME': 'INDUSTRY_NAME_SW2'}), on=['TICKER_SYMBOL'], how='left')
            fund_holding = fund_holding.merge(self.stock_industry_sw[self.stock_industry_sw['INDUSTRY_TYPE'] == 3][['TICKER_SYMBOL', 'INDUSTRY_NAME']].rename(columns={'INDUSTRY_NAME': 'INDUSTRY_NAME_SW3'}), on=['TICKER_SYMBOL'], how='left')
            fund_name = self.all_fund[self.all_fund['FUND_CODE'] == fund_code]['FUND_NAME'].values[0]
            fund_holding['FUND_NAME'] = fund_name
            fund_holding = fund_holding[['FUND_NAME', 'INDUSTRY_NAME_SW1', 'INDUSTRY_NAME_SW2', 'INDUSTRY_NAME_SW3', 'TICKER_SYMBOL', 'SEC_SHORT_NAME'] + fund_holding_date_columns]
            fund_holding.columns = ['基金名称', '申万一级行业', '申万二级行业', '申万三级行业', '股票代码', '股票名称'] + fund_holding_date_columns
            fund_holding.loc[fund_holding.shape[0]] = [np.nan] * fund_holding.shape[1]
            fund_holding_list.append(fund_holding)

            if fund_code == self.target_fund_code:
                target_start_date = self.all_fund_holding[self.all_fund_holding['FUND_CODE'] == self.target_fund_code]['REPORT_DATE'].min()
                target_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == self.target_fund_code]['FUND_NAME'].values[0]
                for idx, stock in enumerate(fund_holding[['股票代码', fund_holding_date_columns[-1]]].dropna()['股票代码'].unique().tolist()[:10]):
                    # 个股仓位变动图
                    stock_daily_k = self.all_stock_daily_k[self.all_stock_daily_k['TICKER_SYMBOL'] == stock]
                    stock_daily_k['TRADE_DATE'] = stock_daily_k['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    stock_daily_k = stock_daily_k.sort_values('TRADE_DATE')
                    stock_name = stock_daily_k['SEC_SHORT_NAME'].values[0]
                    stock_valuation = self.all_stock_valuation[self.all_stock_valuation['TICKER_SYMBOL'] == stock]
                    stock_valuation['TRADE_DATE'] = stock_valuation['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    stock_valuation = stock_valuation.sort_values('TRADE_DATE')
                    stock_finance = self.all_stock_finance[self.all_stock_finance['TICKER_SYMBOL'] == stock]
                    stock_finance = stock_finance.sort_values(['END_DATE', 'PUBLISH_DATE']).drop_duplicates(['END_DATE'], keep='last')
                    stock_finance = stock_finance.merge(self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'END_DATE'}), on=['END_DATE'], how='left')
                    stock_finance['TRADE_DATE'] = stock_finance['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    stock_finance = stock_finance.sort_values('TRADE_DATE')
                    stock_holding_disp = self.all_fund_holding[(self.all_fund_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_holding['TICKER_SYMBOL'] == stock)]
                    stock_holding_disp = stock_holding_disp[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE')
                    stock_holding_disp['MV_IN_NA'] = stock_holding_disp['MV_IN_NA'].apply(lambda x: round(x, 2))
                    stock_holding_disp.columns = [target_fund_name]
                    disp_start_date = target_start_date if self.begin_date is None else self.begin_date
                    disp_dates = self.all_report_df[self.all_report_df['REPORT_DATE'] >= disp_start_date]['REPORT_DATE'].unique().tolist()
                    stock_holding_disp = stock_holding_disp.reindex(disp_dates)
                    stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding_disp[target_fund_name].isnull()), target_fund_name] = 0.00
                    stock_holding_disp = pd.concat([stock_holding_disp, self.all_fund_holding_min[self.all_fund_holding_min['FUND_CODE'] == self.target_fund_code][['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE')], axis=1)
                    stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[target_fund_name].isnull()), target_fund_name] = stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[target_fund_name].isnull()), 'MV_IN_NA_MIN']
                    stock_holding_disp = stock_holding_disp.drop('MV_IN_NA_MIN', axis=1)
                    stock_holding_disp.loc[stock_holding_disp.index < target_start_date, target_fund_name] = '--'
                    stock_holding_disp[target_fund_name] = stock_holding_disp[target_fund_name].astype(str)
                    stock_holding_disp = stock_holding_disp.reset_index()
                    stock_holding_disp = stock_holding_disp.merge(self.all_calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                    stock_holding_disp['TRADE_DATE'] = stock_holding_disp['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    stock_holding_disp = stock_holding_disp.sort_values('TRADE_DATE')
                    stock_holding_disp_quancang = stock_holding_disp.loc[stock_holding_disp['REPORT_DATE'].str.slice(4, 6).isin(['06', '12'])]
                    stock_holding_disp_quancang[target_fund_name] = stock_holding_disp_quancang[target_fund_name].replace('--', np.nan)
                    stock_holding_disp_quancang[target_fund_name] = stock_holding_disp_quancang[target_fund_name].astype(float)
                    stock_holding_disp_quancang = stock_holding_disp_quancang.rename(columns={target_fund_name: target_fund_name + '（全仓）'})
                    all_report_trade_df = self.all_report_trade_df[['TRADE_DATE']]
                    all_report_trade_df['TRADE_DATE'] = all_report_trade_df['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    all_report_trade_df['IS_QUARTER_END'] = 1
                    data = stock_daily_k[stock_daily_k['TRADE_DATE'] >= datetime.strptime(disp_start_date, '%Y%m%d') - timedelta(30)][['TRADE_DATE', 'CLOSE_PRICE']]
                    data = data.merge(stock_valuation[['TRADE_DATE', 'PE_TTM']], on=['TRADE_DATE'], how='left')
                    data = data.merge(stock_finance[['TRADE_DATE', 'ROE_TTM']], on=['TRADE_DATE'], how='left')
                    data = data.merge(stock_holding_disp[['TRADE_DATE', target_fund_name]], on=['TRADE_DATE'], how='left')
                    data = data.merge(stock_holding_disp_quancang[['TRADE_DATE', target_fund_name + '（全仓）']], on=['TRADE_DATE'], how='left')
                    data = data.merge(all_report_trade_df[['TRADE_DATE', 'IS_QUARTER_END']], on=['TRADE_DATE'], how='left')
                    data['INDEX'] = range(len(data))
                    data['TRADE_DATE_DISP'] = data['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d')[:6])
                    fig, ax = plt.subplots(2, 1, figsize=(25, 10))
                    bar_width = len(data) / len(data.dropna(subset=[target_fund_name])) / 10.0
                    ax0_right = ax[0].twinx()
                    ax[0].plot(data['INDEX'], data['CLOSE_PRICE'], color='#AFDEFA', label='{0}（元）'.format(stock_name))
                    ax0_right.bar(data['INDEX'], data[target_fund_name + '（全仓）'], width=bar_width, color='#C94649', label='占净值比（%）')
                    ax0_right.set_ylim([0, 50])
                    ax[0].scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_PRICE'], marker='.', color='#F04950', label='{0}中该个股持仓占净值比（%）'.format(target_fund_name))
                    for x, y, value in zip(list(data.dropna(subset=[target_fund_name])['INDEX']), list(data.dropna(subset=[target_fund_name])['CLOSE_PRICE']), list(data.dropna(subset=[target_fund_name])[target_fund_name])):
                        ax[0].text(x, y * 1.1, value, ha='center', va='bottom', color='#F04950')
                    ax[0].scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_PRICE'], marker='.', color='#959595')
                    plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                    ax[0].set_xlabel('')
                    ax[0].set_ylabel('')
                    ax0_right.set_ylabel('')
                    ax[0].legend(loc=2)
                    ax0_right.legend(loc=1)
                    ax0_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                    ax1_right = ax[1].twinx()
                    ax[1].bar([i - bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['PE_TTM']), width=bar_width, color='#C94649', label='PE_TTM')
                    ax1_right.bar([i + bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['ROE_TTM']), width=bar_width, color='#8588B7', label='ROE_TTM')
                    plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                    ax[1].set_xlabel('')
                    ax[1].set_ylabel('')
                    ax1_right.set_ylabel('')
                    ax[1].legend(loc=2)
                    ax1_right.legend(loc=1)
                    ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                    plt.tight_layout()
                    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/{1}_{2}.png'.format(self.target_fund_code, stock_name, target_fund_name))
                    worksheet_h.insert_image('A{0}'.format(idx + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/{1}_{2}.png'.format(self.target_fund_code, stock_name, target_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                    ########################################
                    hypre_link = r'=HYPERLINK("#' + '个股仓位变动图' + '!' + 'A{0}'.format(idx + 1) + '\",\"' + fund_holding.iloc[idx, 5] + '")'
                    fund_holding.iloc[idx, 5] = hypre_link

        fund_holding = pd.concat(fund_holding_list)
        fund_holding.loc[:, fund_holding.shape[1]] = [np.nan] * fund_holding.shape[0]
        fund_holding = fund_holding.rename(columns={list(fund_holding.columns)[-1]: ''})
        fund_holding = fund_holding.reset_index().drop('index', axis=1)

        target_start_date = self.all_fund_holding[self.all_fund_holding['FUND_CODE'] == self.target_fund_code]['REPORT_DATE'].min()
        target_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == self.target_fund_code]['FUND_NAME'].values[0]
        target_fund_stocks = self.all_fund_holding[(self.all_fund_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_holding['REPORT_DATE'].isin(sorted(self.all_fund_holding['REPORT_DATE'].unique().tolist())[-12:])) & (self.all_fund_holding['MV_IN_NA'] >= 4.0)]['TICKER_SYMBOL'].unique().tolist()
        same_stocks_list = []
        stock_holding_list = []
        for j, compare_fund_code in enumerate(self.compare_fund_code_list):
            compare_fund_stocks = self.all_fund_holding[(self.all_fund_holding['FUND_CODE'] == compare_fund_code) & (self.all_fund_holding['REPORT_DATE'].isin(sorted(self.all_fund_holding['REPORT_DATE'].unique().tolist())[-12:])) & (self.all_fund_holding['MV_IN_NA'] >= 4.0)][['TICKER_SYMBOL']].drop_duplicates()
            compare_fund_stocks = compare_fund_stocks[compare_fund_stocks['TICKER_SYMBOL'].isin(target_fund_stocks)]
            compare_fund_stocks = compare_fund_stocks.merge(self.all_stock_name[['TICKER_SYMBOL', 'SEC_SHORT_NAME']], on=['TICKER_SYMBOL'], how='left')
            compare_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == compare_fund_code]['FUND_NAME'].values[0]
            compare_fund_stocks['FUND_NAME'] = compare_fund_name
            compare_fund_stocks = compare_fund_stocks[['FUND_NAME', 'TICKER_SYMBOL', 'SEC_SHORT_NAME']]
            compare_fund_stocks.columns = ['基金名称', '与{0}同持仓股票代码'.format(target_fund_name), '与{0}同持仓股票名称'.format(target_fund_name)]
            compare_fund_stocks = compare_fund_stocks.reset_index().drop('index', axis=1)
            same_stocks_list.append(compare_fund_stocks)

            compare_start_date = self.all_fund_holding[self.all_fund_holding['FUND_CODE'] == compare_fund_code]['REPORT_DATE'].min()
            compare_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == compare_fund_code]['FUND_NAME'].values[0]
            disp_start_date = min(target_start_date, compare_start_date) if self.begin_date is None else self.begin_date
            disp_dates = self.all_report_df[self.all_report_df['REPORT_DATE'] >= disp_start_date]['REPORT_DATE'].unique().tolist()
            for i, stock in enumerate(compare_fund_stocks['与{0}同持仓股票代码'.format(target_fund_name)].unique().tolist()):
                target_holding = self.all_fund_holding[(self.all_fund_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_holding['TICKER_SYMBOL'] == stock)]
                target_holding = target_holding[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE').rename(columns={'MV_IN_NA': target_fund_name})
                target_holding[target_fund_name] = target_holding[target_fund_name].apply(lambda x: round(x, 2))
                compare_holding = self.all_fund_holding[(self.all_fund_holding['FUND_CODE'] == compare_fund_code) & (self.all_fund_holding['TICKER_SYMBOL'] == stock)]
                compare_holding = compare_holding[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE').rename(columns={'MV_IN_NA': compare_fund_name})
                compare_holding[compare_fund_name] = compare_holding[compare_fund_name].apply(lambda x: round(x, 2))
                stock_holding = target_holding.merge(compare_holding, left_index=True, right_index=True, how='outer')
                stock_holding = stock_holding.reindex(disp_dates)

                ########################################
                # 个股仓位变动比较图
                stock_daily_k = self.all_stock_daily_k[self.all_stock_daily_k['TICKER_SYMBOL'] == stock]
                stock_daily_k['TRADE_DATE'] = stock_daily_k['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                stock_daily_k = stock_daily_k.sort_values('TRADE_DATE')
                stock_name = stock_daily_k['SEC_SHORT_NAME'].values[0]
                stock_valuation = self.all_stock_valuation[self.all_stock_valuation['TICKER_SYMBOL'] == stock]
                stock_valuation['TRADE_DATE'] = stock_valuation['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                stock_valuation = stock_valuation.sort_values('TRADE_DATE')
                stock_finance = self.all_stock_finance[self.all_stock_finance['TICKER_SYMBOL'] == stock]
                stock_finance = stock_finance.sort_values(['END_DATE', 'PUBLISH_DATE']).drop_duplicates(['END_DATE'], keep='last')
                stock_finance = stock_finance.merge(self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'END_DATE'}), on=['END_DATE'], how='left')
                stock_finance['TRADE_DATE'] = stock_finance['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                stock_finance = stock_finance.sort_values('TRADE_DATE')
                stock_holding_disp = stock_holding.copy(deep=True)
                stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding_disp[target_fund_name].isnull()), target_fund_name] = 0.00
                stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding_disp[compare_fund_name].isnull()), compare_fund_name] = 0.00
                stock_holding_disp = pd.concat([stock_holding_disp, self.all_fund_holding_min[self.all_fund_holding_min['FUND_CODE'] == self.target_fund_code][['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE')], axis=1)
                stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[target_fund_name].isnull()), target_fund_name] = stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[target_fund_name].isnull()), 'MV_IN_NA_MIN']
                stock_holding_disp = stock_holding_disp.drop('MV_IN_NA_MIN', axis=1)
                stock_holding_disp = pd.concat([stock_holding_disp, self.all_fund_holding_min[self.all_fund_holding_min['FUND_CODE'] == compare_fund_code][['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE')], axis=1)
                stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[compare_fund_name].isnull()), compare_fund_name] = stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[compare_fund_name].isnull()), 'MV_IN_NA_MIN']
                stock_holding_disp = stock_holding_disp.drop('MV_IN_NA_MIN', axis=1)
                stock_holding_disp.loc[stock_holding_disp.index < target_start_date, target_fund_name] = '--'
                stock_holding_disp.loc[stock_holding_disp.index < compare_start_date, compare_fund_name] = '--'
                stock_holding_disp[target_fund_name] = stock_holding_disp[target_fund_name].astype(str)
                stock_holding_disp[compare_fund_name] = stock_holding_disp[compare_fund_name].astype(str)
                stock_holding_disp = stock_holding_disp.reset_index()
                stock_holding_disp = stock_holding_disp.merge(self.all_calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                stock_holding_disp['TRADE_DATE'] = stock_holding_disp['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                stock_holding_disp = stock_holding_disp.sort_values('TRADE_DATE')
                stock_holding_disp_quancang = stock_holding_disp.loc[stock_holding_disp['REPORT_DATE'].str.slice(4, 6).isin(['06', '12'])]
                stock_holding_disp_quancang[target_fund_name] = stock_holding_disp_quancang[target_fund_name].replace('--', np.nan)
                stock_holding_disp_quancang[compare_fund_name] = stock_holding_disp_quancang[compare_fund_name].replace('--', np.nan)
                stock_holding_disp_quancang[target_fund_name] = stock_holding_disp_quancang[target_fund_name].astype(float)
                stock_holding_disp_quancang[compare_fund_name] = stock_holding_disp_quancang[compare_fund_name].astype(float)
                stock_holding_disp_quancang = stock_holding_disp_quancang.rename(columns={target_fund_name: target_fund_name + '（全仓）', compare_fund_name: compare_fund_name + '（全仓）'})
                all_report_trade_df = self.all_report_trade_df[['TRADE_DATE']]
                all_report_trade_df['TRADE_DATE'] = all_report_trade_df['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                all_report_trade_df['IS_QUARTER_END'] = 1
                data = stock_daily_k[stock_daily_k['TRADE_DATE'] >= datetime.strptime(disp_start_date, '%Y%m%d') - timedelta(30)][['TRADE_DATE', 'CLOSE_PRICE']]
                data = data.merge(stock_valuation[['TRADE_DATE', 'PE_TTM']], on=['TRADE_DATE'], how='left')
                data = data.merge(stock_finance[['TRADE_DATE', 'ROE_TTM']], on=['TRADE_DATE'], how='left')
                data = data.merge(stock_holding_disp[['TRADE_DATE', target_fund_name, compare_fund_name]], on=['TRADE_DATE'], how='left')
                data = data.merge(stock_holding_disp_quancang[['TRADE_DATE', target_fund_name + '（全仓）', compare_fund_name + '（全仓）']], on=['TRADE_DATE'], how='left')
                data = data.merge(all_report_trade_df[['TRADE_DATE', 'IS_QUARTER_END']], on=['TRADE_DATE'], how='left')
                data['INDEX'] = range(len(data))
                data['TRADE_DATE_DISP'] = data['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d')[:6])
                fig, ax = plt.subplots(2, 1, figsize=(25, 10))
                bar_width = len(data) / len(data.dropna(subset=[target_fund_name])) / 10.0
                ax0_right = ax[0].twinx()
                ax[0].plot(data['INDEX'], data['CLOSE_PRICE'], color='#AFDEFA', label='{0}（元）'.format(stock_name))
                ax0_right.bar([i - bar_width / 2.0 for i in data['INDEX']], data[target_fund_name + '（全仓）'], width=bar_width, color='#C94649')
                ax0_right.bar([i + bar_width / 2.0 for i in data['INDEX']], data[compare_fund_name + '（全仓）'], width=bar_width, color='#8588B7')
                ax0_right.set_ylim([0, 50])
                ax[0].scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_PRICE'], marker='.', color='#F04950', label='{0}中该个股持仓占净值比（%）'.format(target_fund_name))
                ax[0].scatter(data.dropna(subset=[compare_fund_name])['INDEX'], data.dropna(subset=[compare_fund_name])['CLOSE_PRICE'], marker='.', color='#6268A2', label='{0}中该个股持仓占净值比（%）'.format(compare_fund_name))
                for x, y, value in zip(list(data.dropna(subset=[target_fund_name])['INDEX']), list(data.dropna(subset=[target_fund_name])['CLOSE_PRICE']), list(data.dropna(subset=[target_fund_name])[target_fund_name])):
                    ax[0].text(x, y * 1.1, value, ha='center', va='bottom', color='#F04950')
                for x, y, value in zip(list(data.dropna(subset=[compare_fund_name])['INDEX']), list(data.dropna(subset=[compare_fund_name])['CLOSE_PRICE']), list(data.dropna(subset=[compare_fund_name])[compare_fund_name])):
                    ax[0].text(x, y * 0.9, value, ha='center', va='bottom', color='#6268A2')
                ax[0].scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_PRICE'], marker='.', color='#959595')
                ax[0].scatter(data.dropna(subset=[compare_fund_name])['INDEX'], data.dropna(subset=[compare_fund_name])['CLOSE_PRICE'], marker='.', color='#959595')
                plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'],  labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                ax[0].set_xlabel('')
                ax[0].set_ylabel('')
                ax0_right.set_ylabel('占净值比（%）')
                ax[0].legend(loc=2)
                ax0_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                ax1_right = ax[1].twinx()
                ax[1].bar([i - bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['PE_TTM']), width=bar_width, color='#C94649', label='PE_TTM')
                ax1_right.bar([i + bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['ROE_TTM']), width=bar_width, color='#8588B7', label='ROE_TTM')
                plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'],  labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                ax[1].set_xlabel('')
                ax[1].set_ylabel('')
                ax1_right.set_ylabel('')
                ax[1].legend(loc=2)
                ax1_right.legend(loc=1)
                ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                plt.tight_layout()
                plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, stock_name, target_fund_name, compare_fund_name))
                worksheet_hc.insert_image('{0}{1}'.format(cell_dic[j + 1], i + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, stock_name, target_fund_name, compare_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                ########################################

                stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding[target_fund_name].isnull()), target_fund_name] = 0.00
                stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding[compare_fund_name].isnull()), compare_fund_name] = 0.00
                stock_holding = pd.concat([stock_holding, self.all_fund_holding_min[self.all_fund_holding_min['FUND_CODE'] == self.target_fund_code][['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE')], axis=1)
                stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding[target_fund_name].isnull()), target_fund_name] = stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding[target_fund_name].isnull()), 'MV_IN_NA_MIN']
                stock_holding = stock_holding.drop('MV_IN_NA_MIN', axis=1)
                stock_holding = pd.concat([stock_holding, self.all_fund_holding_min[self.all_fund_holding_min['FUND_CODE'] == compare_fund_code][['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE')], axis=1)
                stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding[compare_fund_name].isnull()), compare_fund_name] = stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding[compare_fund_name].isnull()), 'MV_IN_NA_MIN']
                stock_holding = stock_holding.drop('MV_IN_NA_MIN', axis=1)
                stock_holding.loc[stock_holding.index < target_start_date, target_fund_name] = '--'
                stock_holding.loc[stock_holding.index < compare_start_date, compare_fund_name] = '--'
                stock_holding = stock_holding.sort_index().T.reset_index().rename(columns={'index': '基金名称'})
                stock_holding_columns = list(stock_holding.columns)
                stock_holding['TICKER_SYMBOL'] = stock
                stock_holding = stock_holding.merge(self.all_stock_name[['TICKER_SYMBOL', 'SEC_SHORT_NAME']], on=['TICKER_SYMBOL'], how='left')
                stock_holding = stock_holding[['SEC_SHORT_NAME'] + stock_holding_columns]
                stock_holding.columns = ['股票名称'] + stock_holding_columns
                stock_holding['个股仓位变动比较图'] = ''
                stock_holding.loc[stock_holding.shape[0]] = [np.nan] * stock_holding.shape[1]
                stock_holding = stock_holding.T.reset_index().T
                hypre_link = r'=HYPERLINK("#' + '个股仓位变动比较图' + '!' + '{0}{1}'.format(cell_dic[j + 1], i + 1) + '\",\"' + '个股仓位变动比较图' + '")'
                stock_holding.iloc[0, -1] = hypre_link
                stock_holding_list.append(stock_holding)

        if len(self.compare_fund_code_list) > 0:
            same_stocks = pd.concat(same_stocks_list, axis=1)
            same_stocks.loc[:, same_stocks.shape[1]] = [np.nan] * same_stocks.shape[0]
            same_stocks = same_stocks.rename(columns={list(same_stocks.columns)[-1]: ''})
            same_stocks = same_stocks.reset_index().drop('index', axis=1)
            stock_holding = pd.concat(stock_holding_list)
            stock_holding.columns = [''] * stock_holding.shape[1]
            stock_holding = stock_holding.reset_index().drop('index', axis=1)
            stock_holding_compare = pd.concat([fund_holding, same_stocks, stock_holding], axis=1)
        else:
            stock_holding_compare = pd.concat([fund_holding], axis=1)
        return stock_holding_compare

    def get_industry_holding_compare(self, worksheet_h, worksheet_hc):
        target_start_date = self.all_fund_industry_holding[self.all_fund_industry_holding['FUND_CODE'] == self.target_fund_code]['REPORT_DATE'].min()
        all_fund_industry_holding_cut = self.all_fund_industry_holding[self.all_fund_industry_holding['REPORT_DATE'] >= target_start_date]
        fund_industry_holding_list = []
        for fund_code in [self.target_fund_code] + self.compare_fund_code_list:
            # 行业仓位情况
            fund_industry_holding = all_fund_industry_holding_cut[all_fund_industry_holding_cut['FUND_CODE'] == fund_code]
            fund_industry_holding = fund_industry_holding.pivot(index='INDUSTRY_NAME', columns='REPORT_DATE', values='MV_IN_NA')
            fund_industry_holding_date_columns = sorted(list(fund_industry_holding.columns))
            industry_rank_list = list(fund_industry_holding[fund_industry_holding_date_columns[-1]].sort_values(ascending=False).index)
            fund_industry_holding = fund_industry_holding.reset_index()
            fund_industry_holding['INDUSTRY_NAME'] = fund_industry_holding['INDUSTRY_NAME'].astype('category')
            fund_industry_holding['INDUSTRY_NAME'].cat.reorder_categories(industry_rank_list, inplace=True)
            fund_industry_holding = fund_industry_holding.sort_values('INDUSTRY_NAME')
            fund_name = self.all_fund[self.all_fund['FUND_CODE'] == fund_code]['FUND_NAME'].values[0]
            fund_industry_holding['FUND_NAME'] = fund_name
            fund_industry_holding = fund_industry_holding[['FUND_NAME', 'INDUSTRY_NAME'] + fund_industry_holding_date_columns]
            fund_industry_holding.columns = ['基金名称', '申万一级行业'] + fund_industry_holding_date_columns
            fund_industry_holding.loc[fund_industry_holding.shape[0]] = [np.nan] * fund_industry_holding.shape[1]
            fund_industry_holding_list.append(fund_industry_holding)

            if fund_code == self.target_fund_code:
                target_start_date = self.all_fund_industry_holding[self.all_fund_industry_holding['FUND_CODE'] == self.target_fund_code]['REPORT_DATE'].min()
                target_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == self.target_fund_code]['FUND_NAME'].values[0]
                for idx, industry in enumerate(fund_industry_holding[['申万一级行业', fund_industry_holding_date_columns[-1]]].dropna()['申万一级行业'].unique().tolist()):
                    # 行业仓位变动图
                    industry_daily_k = self.all_industry_daily_k[self.all_industry_daily_k['INDUSTRY_NAME'] == industry]
                    industry_daily_k['TRADE_DATE'] = industry_daily_k['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    industry_daily_k = industry_daily_k.sort_values('TRADE_DATE')
                    industry_valuation = self.all_industry_valuation[self.all_industry_valuation['INDUSTRY_NAME'] == industry]
                    industry_valuation = industry_valuation.merge(self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                    industry_valuation['TRADE_DATE'] = industry_valuation['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    industry_valuation = industry_valuation.sort_values('TRADE_DATE')
                    industry_finance = self.all_industry_finance[self.all_industry_finance['INDUSTRY_NAME'] == industry]
                    industry_finance = industry_finance.merge(self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                    industry_finance['TRADE_DATE'] = industry_finance['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    industry_finance = industry_finance.sort_values('TRADE_DATE')
                    industry_holding_disp = self.all_fund_industry_holding[(self.all_fund_industry_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_industry_holding['INDUSTRY_NAME'] == industry)]
                    industry_holding_disp = industry_holding_disp[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE')
                    industry_holding_disp['MV_IN_NA'] = industry_holding_disp['MV_IN_NA'].apply(lambda x: round(x, 2))
                    industry_holding_disp.columns = [target_fund_name]
                    disp_start_date = target_start_date if self.begin_date is None else self.begin_date
                    disp_dates = self.all_report_df[self.all_report_df['REPORT_DATE'] >= disp_start_date]['REPORT_DATE'].unique().tolist()
                    industry_holding_disp = industry_holding_disp.reindex(disp_dates)
                    industry_holding_disp[target_fund_name] = industry_holding_disp[target_fund_name].fillna(0.00)
                    industry_holding_disp.loc[ industry_holding_disp.index.str.slice(4, 6).isin(['03', '09']), target_fund_name] = industry_holding_disp.loc[industry_holding_disp.index.str.slice(4, 6).isin(['03', '09']), target_fund_name].apply(lambda x: '>={0}'.format(x))
                    industry_holding_disp.loc[industry_holding_disp.index < target_start_date, target_fund_name] = '--'
                    industry_holding_disp[target_fund_name] = industry_holding_disp[target_fund_name].astype(str)
                    industry_holding_disp = industry_holding_disp.reset_index()
                    industry_holding_disp = industry_holding_disp.merge(self.all_calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                    industry_holding_disp['TRADE_DATE'] = industry_holding_disp['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    industry_holding_disp = industry_holding_disp.sort_values('TRADE_DATE')
                    industry_holding_disp_quancang = industry_holding_disp.loc[industry_holding_disp['REPORT_DATE'].str.slice(4, 6).isin(['06', '12'])]
                    industry_holding_disp_quancang[target_fund_name] = industry_holding_disp_quancang[target_fund_name].replace('--', np.nan)
                    industry_holding_disp_quancang[target_fund_name] = industry_holding_disp_quancang[target_fund_name].astype(float)
                    industry_holding_disp_quancang = industry_holding_disp_quancang.rename(columns={target_fund_name: target_fund_name + '（全仓）'})
                    all_report_trade_df = self.all_report_trade_df[['TRADE_DATE']]
                    all_report_trade_df['TRADE_DATE'] = all_report_trade_df['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                    all_report_trade_df['IS_QUARTER_END'] = 1
                    data = industry_daily_k[industry_daily_k['TRADE_DATE'] >= datetime.strptime(disp_start_date, '%Y%m%d') - timedelta(30)][['TRADE_DATE', 'CLOSE_INDEX']]
                    data = data.merge(industry_valuation[['TRADE_DATE', 'PE_TTM']], on=['TRADE_DATE'], how='left')
                    data = data.merge(industry_finance[['TRADE_DATE', 'ROE_TTM']], on=['TRADE_DATE'], how='left')
                    data = data.merge(industry_holding_disp[['TRADE_DATE', target_fund_name]], on=['TRADE_DATE'], how='left')
                    data = data.merge(industry_holding_disp_quancang[['TRADE_DATE', target_fund_name + '（全仓）']], on=['TRADE_DATE'], how='left')
                    data = data.merge(all_report_trade_df[['TRADE_DATE', 'IS_QUARTER_END']], on=['TRADE_DATE'], how='left')
                    data['INDEX'] = range(len(data))
                    data['TRADE_DATE_DISP'] = data['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d')[:6])
                    fig, ax = plt.subplots(figsize=(25, 10))
                    bar_width = len(data) / len(data.dropna(subset=[target_fund_name])) / 10.0
                    ax_right = ax.twinx()
                    ax.plot(data['INDEX'], data['CLOSE_INDEX'], color='#AFDEFA', label='{0}（元）'.format(industry))
                    ax_right.bar(data['INDEX'], data[target_fund_name + '（全仓）'], width=bar_width, color='#C94649', label='占净值比（%）')
                    ax_right.set_ylim([0, 50])
                    ax.scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_INDEX'], marker='.', color='#F04950', label='{0}中该行业持仓占净值比（%）'.format(target_fund_name))
                    for x, y, value in zip(list(data.dropna(subset=[target_fund_name])['INDEX']), list(data.dropna(subset=[target_fund_name])['CLOSE_INDEX']), list(data.dropna(subset=[target_fund_name])[target_fund_name])):
                        ax.text(x, y * 1.1, value, ha='center', va='bottom', color='#F04950')
                    ax.scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_INDEX'], marker='.', color='#959595')
                    plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                    ax.set_xlabel('')
                    ax.set_ylabel('')
                    ax_right.set_ylabel('')
                    ax_right.set_ylim([0, 100])
                    ax.legend(loc=2)
                    ax_right.legend(loc=1)
                    ax_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                    # ax1_right = ax[1].twinx()
                    # ax[1].bar([i - bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['PE_TTM']), width=bar_width, color='#C94649', label='PE_TTM')
                    # ax1_right.bar([i + bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['ROE_TTM']), width=bar_width, color='#8588B7', label='ROE_TTM')
                    # plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                    # ax[1].set_xlabel('')
                    # ax[1].set_ylabel('')
                    # ax1_right.set_ylabel('')
                    # ax[1].legend(loc=2)
                    # ax1_right.legend(loc=1)
                    # ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                    plt.tight_layout()
                    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/{1}_{2}.png'.format(self.target_fund_code, industry, target_fund_name))
                    worksheet_h.insert_image('A{0}'.format(idx + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/{1}_{2}.png'.format(self.target_fund_code, industry, target_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                    ########################################
                    hypre_link = r'=HYPERLINK("#' + '行业仓位变动图' + '!' + 'A{0}'.format(idx + 1) + '\",\"' + fund_industry_holding.iloc[idx, 1] + '")'
                    fund_industry_holding.iloc[idx, 1] = hypre_link

        fund_industry_holding = pd.concat(fund_industry_holding_list)
        fund_industry_holding.loc[:, fund_industry_holding.shape[1]] = [np.nan] * fund_industry_holding.shape[0]
        fund_industry_holding = fund_industry_holding.rename(columns={list(fund_industry_holding.columns)[-1]: ''})
        fund_industry_holding = fund_industry_holding.reset_index().drop('index', axis=1)

        target_start_date = self.all_fund_industry_holding[self.all_fund_industry_holding['FUND_CODE'] == self.target_fund_code]['REPORT_DATE'].min()
        target_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == self.target_fund_code]['FUND_NAME'].values[0]
        target_fund_industry = self.all_fund_industry_holding[(self.all_fund_industry_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_industry_holding['REPORT_DATE'].isin(sorted(self.all_fund_industry_holding['REPORT_DATE'].unique().tolist())[-12:])) & (self.all_fund_industry_holding['MV_IN_NA'] >= 10)]['INDUSTRY_NAME'].unique().tolist()
        same_industry_list = []
        industry_holding_list = []
        for j, compare_fund_code in enumerate(self.compare_fund_code_list):
            compare_fund_industry = self.all_fund_industry_holding[(self.all_fund_industry_holding['FUND_CODE'] == compare_fund_code) & (self.all_fund_industry_holding['REPORT_DATE'].isin(sorted(self.all_fund_industry_holding['REPORT_DATE'].unique().tolist())[-12:])) & (self.all_fund_industry_holding['MV_IN_NA'] >= 10)][['INDUSTRY_NAME']].drop_duplicates()
            compare_fund_industry = compare_fund_industry[compare_fund_industry['INDUSTRY_NAME'].isin(target_fund_industry)]
            compare_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == compare_fund_code]['FUND_NAME'].values[0]
            compare_fund_industry['FUND_NAME'] = compare_fund_name
            compare_fund_industry = compare_fund_industry[['FUND_NAME', 'INDUSTRY_NAME']]
            compare_fund_industry.columns = ['基金名称', '与{0}同持仓行业名称'.format(target_fund_name)]
            compare_fund_industry = compare_fund_industry.reset_index().drop('index', axis=1)
            same_industry_list.append(compare_fund_industry)

            compare_start_date = self.all_fund_industry_holding[self.all_fund_industry_holding['FUND_CODE'] == compare_fund_code]['REPORT_DATE'].min()
            compare_fund_name = self.all_fund[self.all_fund['FUND_CODE'] == compare_fund_code]['FUND_NAME'].values[0]
            disp_start_date = min(target_start_date, compare_start_date) if self.begin_date is None else self.begin_date
            disp_dates = self.all_report_df[self.all_report_df['REPORT_DATE'] >= disp_start_date]['REPORT_DATE'].unique().tolist()
            for i, industry in enumerate(compare_fund_industry['与{0}同持仓行业名称'.format(target_fund_name)].unique().tolist()):
                target_holding_industry = self.all_fund_industry_holding[(self.all_fund_industry_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_industry_holding['INDUSTRY_NAME'] == industry)]
                target_holding_industry = target_holding_industry[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE').rename(columns={'MV_IN_NA': target_fund_name})
                target_holding_industry[target_fund_name] = target_holding_industry[target_fund_name].apply(lambda x: round(x, 2))
                compare_holding_industry = self.all_fund_industry_holding[(self.all_fund_industry_holding['FUND_CODE'] == compare_fund_code) & (self.all_fund_industry_holding['INDUSTRY_NAME'] == industry)]
                compare_holding_industry = compare_holding_industry[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE').rename(columns={'MV_IN_NA': compare_fund_name})
                compare_holding_industry[compare_fund_name] = compare_holding_industry[compare_fund_name].apply(lambda x: round(x, 2))
                industry_holding = target_holding_industry.merge(compare_holding_industry, left_index=True, right_index=True, how='outer')
                industry_holding = industry_holding.reindex(disp_dates)

                ########################################
                # 行业仓位变动比较图
                industry_daily_k = self.all_industry_daily_k[self.all_industry_daily_k['INDUSTRY_NAME'] == industry]
                industry_daily_k['TRADE_DATE'] = industry_daily_k['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                industry_daily_k = industry_daily_k.sort_values('TRADE_DATE')
                industry_valuation = self.all_industry_valuation[self.all_industry_valuation['INDUSTRY_NAME'] == industry]
                industry_valuation = industry_valuation.merge( self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                industry_valuation['TRADE_DATE'] = industry_valuation['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                industry_valuation = industry_valuation.sort_values('TRADE_DATE')
                industry_finance = self.all_industry_finance[self.all_industry_finance['INDUSTRY_NAME'] == industry]
                industry_finance = industry_finance.merge(self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                industry_finance['TRADE_DATE'] = industry_finance['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                industry_finance = industry_finance.sort_values('TRADE_DATE')
                industry_holding_disp = industry_holding.copy(deep=True)
                industry_holding_disp[target_fund_name] = industry_holding_disp[target_fund_name].fillna(0.00)
                industry_holding_disp[compare_fund_name] = industry_holding_disp[compare_fund_name].fillna(0.00)
                industry_holding_disp.loc[industry_holding_disp.index.str.slice(4, 6).isin(['03', '09']), target_fund_name] = industry_holding_disp.loc[industry_holding_disp.index.str.slice(4, 6).isin(['03', '09']), target_fund_name].apply(lambda x: '>={0}'.format(x))
                industry_holding_disp.loc[industry_holding_disp.index.str.slice(4, 6).isin(['03', '09']), compare_fund_name] = industry_holding_disp.loc[industry_holding_disp.index.str.slice(4, 6).isin(['03', '09']), compare_fund_name].apply(lambda x: '>={0}'.format(x))
                industry_holding_disp.loc[industry_holding_disp.index < target_start_date, target_fund_name] = '--'
                industry_holding_disp.loc[industry_holding_disp.index < compare_start_date, compare_fund_name] = '--'
                industry_holding_disp[target_fund_name] = industry_holding_disp[target_fund_name].astype(str)
                industry_holding_disp[compare_fund_name] = industry_holding_disp[compare_fund_name].astype(str)
                industry_holding_disp = industry_holding_disp.reset_index()
                industry_holding_disp = industry_holding_disp.merge(self.all_calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                industry_holding_disp['TRADE_DATE'] = industry_holding_disp['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                industry_holding_disp = industry_holding_disp.sort_values('TRADE_DATE')
                industry_holding_disp_quancang = industry_holding_disp.loc[industry_holding_disp['REPORT_DATE'].str.slice(4, 6).isin(['06', '12'])]
                industry_holding_disp_quancang[target_fund_name] = industry_holding_disp_quancang[target_fund_name].replace('--', np.nan)
                industry_holding_disp_quancang[compare_fund_name] = industry_holding_disp_quancang[compare_fund_name].replace('--', np.nan)
                industry_holding_disp_quancang[target_fund_name] = industry_holding_disp_quancang[target_fund_name].astype(float)
                industry_holding_disp_quancang[compare_fund_name] = industry_holding_disp_quancang[compare_fund_name].astype(float)
                industry_holding_disp_quancang = industry_holding_disp_quancang.rename(columns={target_fund_name: target_fund_name + '（全仓）', compare_fund_name: compare_fund_name + '（全仓）'})
                all_report_trade_df = self.all_report_trade_df[['TRADE_DATE']]
                all_report_trade_df['TRADE_DATE'] = all_report_trade_df['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
                all_report_trade_df['IS_QUARTER_END'] = 1
                data = industry_daily_k[industry_daily_k['TRADE_DATE'] >= datetime.strptime(disp_start_date, '%Y%m%d') - timedelta(30)][['TRADE_DATE', 'CLOSE_INDEX']]
                data = data.merge(industry_valuation[['TRADE_DATE', 'PE_TTM']], on=['TRADE_DATE'], how='left')
                data = data.merge(industry_finance[['TRADE_DATE', 'ROE_TTM']], on=['TRADE_DATE'], how='left')
                data = data.merge(industry_holding_disp[['TRADE_DATE', target_fund_name, compare_fund_name]], on=['TRADE_DATE'], how='left')
                data = data.merge(industry_holding_disp_quancang[['TRADE_DATE', target_fund_name + '（全仓）', compare_fund_name + '（全仓）']], on=['TRADE_DATE'], how='left')
                data = data.merge(all_report_trade_df[['TRADE_DATE', 'IS_QUARTER_END']], on=['TRADE_DATE'], how='left')
                data['INDEX'] = range(len(data))
                data['TRADE_DATE_DISP'] = data['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d')[:6])
                fig, ax = plt.subplots(1, 1, figsize=(25, 10))
                bar_width = len(data) / len(data.dropna(subset=[target_fund_name])) / 10.0
                ax_right = ax.twinx()
                ax.plot(data['INDEX'], data['CLOSE_INDEX'], color='#AFDEFA', label='{0}（元）'.format(industry))
                ax_right.bar([i - bar_width / 2.0 for i in data['INDEX']], data[target_fund_name + '（全仓）'], width=bar_width, color='#C94649')
                ax_right.bar([i + bar_width / 2.0 for i in data['INDEX']], data[compare_fund_name + '（全仓）'], width=bar_width, color='#8588B7')
                ax_right.set_ylim([0, 50])
                ax.scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_INDEX'], marker='.', color='#F04950', label='{0}中该行业持仓占净值比（%）'.format(target_fund_name))
                ax.scatter(data.dropna(subset=[compare_fund_name])['INDEX'], data.dropna(subset=[compare_fund_name])['CLOSE_INDEX'], marker='.', color='#6268A2', label='{0}中该行业持仓占净值比（%）'.format(compare_fund_name))
                for x, y, value in zip(list(data.dropna(subset=[target_fund_name])['INDEX']), list(data.dropna(subset=[target_fund_name])['CLOSE_INDEX']), list(data.dropna(subset=[target_fund_name])[target_fund_name])):
                    ax.text(x, y * 1.1, value, ha='center', va='bottom', color='#F04950')
                for x, y, value in zip(list(data.dropna(subset=[compare_fund_name])['INDEX']), list(data.dropna(subset=[compare_fund_name])['CLOSE_INDEX']), list(data.dropna(subset=[compare_fund_name])[compare_fund_name])):
                    ax.text(x, y * 0.9, value, ha='center', va='bottom', color='#6268A2')
                ax.scatter(data.dropna(subset=[target_fund_name])['INDEX'], data.dropna(subset=[target_fund_name])['CLOSE_INDEX'], marker='.', color='#959595')
                ax.scatter(data.dropna(subset=[compare_fund_name])['INDEX'], data.dropna(subset=[compare_fund_name])['CLOSE_INDEX'], marker='.', color='#959595')
                plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                ax.set_xlabel('')
                ax.set_ylabel('')
                ax_right.set_ylabel('占净值比（%）')
                ax_right.set_ylim([0, 100])
                ax.legend(loc=2)
                ax_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                # ax1_right = ax[1].twinx()
                # ax[1].bar([i - bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['PE_TTM']), width=bar_width, color='#C94649', label='PE_TTM')
                # ax1_right.bar([i + bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['ROE_TTM']), width=bar_width, color='#8588B7', label='ROE_TTM')
                # plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
                # ax[1].set_xlabel('')
                # ax[1].set_ylabel('')
                # ax1_right.set_ylabel('')
                # ax[1].legend(loc=2)
                # ax1_right.legend(loc=1)
                # ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                plt.tight_layout()
                plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, industry, target_fund_name, compare_fund_name))
                worksheet_hc.insert_image('{0}{1}'.format(cell_dic[j + 1], i + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, industry, target_fund_name, compare_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                ########################################

                industry_holding[target_fund_name] = industry_holding[target_fund_name].fillna(0.00)
                industry_holding[compare_fund_name] = industry_holding[compare_fund_name].fillna(0.00)
                industry_holding.loc[industry_holding.index.str.slice(4, 6).isin(['03', '09']), target_fund_name] = industry_holding.loc[industry_holding.index.str.slice(4, 6).isin(['03', '09']), target_fund_name].apply(lambda x: '>={0}'.format(x))
                industry_holding.loc[industry_holding.index.str.slice(4, 6).isin(['03', '09']), compare_fund_name] = industry_holding.loc[industry_holding.index.str.slice(4, 6).isin(['03', '09']), compare_fund_name].apply(lambda x: '>={0}'.format(x))
                industry_holding.loc[industry_holding.index < target_start_date, target_fund_name] = '--'
                industry_holding.loc[industry_holding.index < compare_start_date, compare_fund_name] = '-'
                industry_holding = industry_holding.sort_index().T.reset_index().rename(columns={'index': '基金名称'})
                industry_holding_columns = list(industry_holding.columns)
                industry_holding['INDUSTRY_NAME'] = industry
                industry_holding = industry_holding[['INDUSTRY_NAME'] + industry_holding_columns]
                industry_holding.columns = ['申万一级行业'] + industry_holding_columns
                industry_holding['行业仓位变动比较图'] = ''
                industry_holding.loc[industry_holding.shape[0]] = [np.nan] * industry_holding.shape[1]
                industry_holding = industry_holding.T.reset_index().T
                hypre_link = r'=HYPERLINK("#' + '行业仓位变动比较图' + '!' + '{0}{1}'.format(cell_dic[j + 1], i + 1) + '\",\"' + '行业仓位变动比较图' + '")'
                industry_holding.iloc[0, -1] = hypre_link
                industry_holding_list.append(industry_holding)

        if len(self.compare_fund_code_list) > 0:
            same_industry = pd.concat(same_industry_list, axis=1)
            same_industry.loc[:, same_industry.shape[1]] = [np.nan] * same_industry.shape[0]
            same_industry = same_industry.rename(columns={list(same_industry.columns)[-1]: ''})
            same_industry = same_industry.reset_index().drop('index', axis=1)
            industry_holding = pd.concat(industry_holding_list)
            industry_holding.columns = [''] * industry_holding.shape[1]
            industry_holding = industry_holding.reset_index().drop('index', axis=1)
            industry_holding_compare = pd.concat([fund_industry_holding, same_industry, industry_holding], axis=1)
        else:
            industry_holding_compare = pd.concat([fund_industry_holding], axis=1)
        return industry_holding_compare


    def get_holding_compare(self):
        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/'):
            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/')
        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/'.format(self.target_fund_code)):
            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/'.format(self.target_fund_code))
        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/'.format(self.target_fund_code)):
            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/'.format(self.target_fund_code))
        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/'.format(self.target_fund_code)):
            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/'.format(self.target_fund_code))
        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/'.format(self.target_fund_code)):
            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/'.format(self.target_fund_code))
        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/'.format(self.target_fund_code)):
            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/'.format(self.target_fund_code))

        excel_writer = pd.ExcelWriter('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/持仓情况.xlsx'.format(self.target_fund_code), engine='xlsxwriter')
        worksheet1 = excel_writer.book.add_worksheet('个股仓位变动图')
        worksheet2 = excel_writer.book.add_worksheet('个股仓位变动比较图')
        worksheet3 = excel_writer.book.add_worksheet('行业仓位变动图')
        worksheet4 = excel_writer.book.add_worksheet('行业仓位变动比较图')
        stock_holding_compare = self.get_stock_holding_compare(worksheet1, worksheet2)
        industry_holding_compare = self.get_industry_holding_compare(worksheet3, worksheet4)
        stock_holding_compare.to_excel(excel_writer, sheet_name='个股持仓情况', index=False)
        industry_holding_compare.to_excel(excel_writer, sheet_name='行业持仓情况', index=False)
        excel_writer.save()
        return

if __name__ == '__main__':
    target_fund_code = '007493'
    compare_fund_code_list = ['001856', '001410', '001975']
    HoldingCompare(target_fund_code, compare_fund_code_list).get_holding_compare()

    # holding_operation('601012', '007493')

    # for stock in ['002141', '300059', '601128', '603816']:
    #     holding_operation(stock, '688888', '20190331')


"""
经济增长类指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class EconomyIncrease:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_eco_increase'

    def get_eco_increase_data(self):
        """
        经济增长类数据：GDP不变价当季同比（实际增速）、GDP现价当季值、GDP平减指数当季同比、失业率、制造业PMI、工业增加值: 当月同比、
                    发电量：当月同比、消费者信心指数、
                    70个大中城市新建商品住宅价格指数:环比、商品房销售面积：累计同比、房地产开发投资完成额：住宅：累计同比、百城住宅价格指数:同比、
                    工业企业：产成品存货：累计同比、工业企业：利润总额：累计同比、工业产能利用率、
                    销量：挖掘机：主要企业：当月同比、
                    九鞅货币条件指数、克强指数：当月值、
                    固定资产投资完成额： 累计同比（总体、制造业、基建、房地产）、中国出口集装箱运价指数(CCFI)、OECD综合领先指标:中国
        """
        index_list = ['M0039354', 'M5567876', 'M5439528', 'M5650805', 'M0017126', 'M0000545',
                      'S0027013', 'M0012303',
                      'S2707412', 'S0073300', 'S0049576', 'S2704485',
                      'M0000561', 'M0000557', 'M5792266',
                      'S6002167',
                      'M6641342', 'M5407921',
                      'M0000273', 'M0000357', 'M5440435', 'M0000449',
                      'S0000066', 'G1000116']
        name_dict = {'M0039354': 'GDP_real', 'M5567876': 'GDP_current_price', 'M5439528': "GDP_deflator",
                     'M5650805': 'unemployment',
                     'M0017126': 'PMI', 'M0000545': "IVA_yoy",
                     'S0027013': 'generating_cap_yoy', 'M0012303': 'Consumer_Index',
                     'S2707412': 'house_price_yoy', 'S0073300': 'house_sell_yoy',
                     'S0049576': "house_investment_yoy", 'S2704485': "hundred_house_price_yoy",
                     'M0000561': 'ind_goods_cum_yoy', 'M0000557': 'ind_income_cum_yoy', 'M5792266': 'ind_cu_rate',
                     'S6002167': 'excavator_yoy',
                     'M6641342': 'jy_curr_index', 'M5407921': 'kq_index',
                     'M0000273': 'FAI_cum_yoy', 'M0000357': 'FAI_mf_cum_yoy',
                     'M5440435': 'FAI_fm_cum_yoy', 'M0000449': 'FAI_re_cum_yoy',
                     'S0000066': 'CCFI', 'G1000116': "OECD_leading_indicator"}

        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch economy increase data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        data['GDP_deflator'] = data['GDP_deflator'].round(2)
        data['IVA_yoy'] = data['IVA_yoy'].round(1)
        data['OECD_leading_indicator'] = data['OECD_leading_indicator'].round(2)
        data['jy_curr_index'] = data['jy_curr_index'].round(4)
        data['kq_index'] = data['kq_index'].round(2)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_eco_increase_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                    create table mac_eco_increase(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    GDP_real decimal(6, 1),
                    GDP_current_price decimal(9, 2),
                    GDP_deflator decimal(4, 2),
                    unemployment decimal(3, 1),
                    PMI decimal(4, 1),
                    IVA_yoy decimal(4, 1),
                    generating_cap_yoy decimal(6, 2),
                    Consumer_Index decimal(6, 2),
                    house_price_yoy decimal(4, 2),
                    house_sell_yoy decimal(5, 2),
                    house_investment_yoy decimal(5, 2),
                    hundred_house_price_yoy decimal(4, 2),
                    ind_goods_cum_yoy decimal(5, 2),
                    ind_income_cum_yoy decimal(5, 2),
                    ind_cu_rate decimal(5, 2),
                    excavator_yoy decimal(5, 2),
                    jy_curr_index decimal(5, 4),
                    kq_index decimal(5, 2),
                    FAI_cum_yoy decimal(5, 2),
                    FAI_mf_cum_yoy decimal(5, 2),
                    FAI_fm_cum_yoy decimal(5, 2),
                    FAI_re_cum_yoy decimal(5, 2),
                    CCFI decimal(6, 2),
                    OECD_leading_indicator decimal(6, 2)) 
                """
            create_table(self.table_name, sql_script)
            data = self.get_eco_increase_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    EconomyIncrease('2005-01-01', '2021-06-10', is_increment=0).get_construct_result()


import datetime
import pandas as pd
import numpy as np
from hbshare.fe.XZ import db_engine
from hbshare.fe.XZ import functionality
from hbshare.fe.mutual_analysis import pool_analysis as pla




util=functionality.Untils()
hbdb=db_engine.HBDB()
localdb=db_engine.PrvFunDB().engine

jjdm_list = util.get_mutual_stock_funds(datetime.datetime.today().strftime('%Y%m%d'))
jjdm_con = util.list_sql_condition(jjdm_list)
sql = "select jjjc,jjdm from st_fund.t_st_gm_jjxx where jjdm in ({0})".format(jjdm_con)
jjdm_name_map = hbdb.db2df(sql, db='funduser')

class Equilibrium:

    @staticmethod
    def ind_equ():

        latest_asofdate=\
            pd.read_sql("select max(asofdate) as asofdate from hbs_industry_property_new ",con=localdb)['asofdate'][0]

        sql="SELECT jjdm,cen_ind_1,asofdate from hbs_industry_property_new where asofdate='{0}' order by cen_ind_1 ".format(latest_asofdate)
        ind_equ=pd.read_sql(sql,con=localdb)

        return ind_equ

    @staticmethod
    def style_equ():

        latest_asofdate = \
        pd.read_sql("select max(asofdate) as asofdate from hbs_style_property ", con=localdb)['asofdate'][0]

        sql="""SELECT jjdm,cen_lv_rank,asofdate 
        from hbs_style_property where asofdate='{0}' order by cen_lv_rank 
        """.format(latest_asofdate)
        style_equ = pd.read_sql(sql, con=localdb)

        return  style_equ

    @staticmethod
    def size_equ():

        latest_asofdate = \
        pd.read_sql("select max(asofdate) as asofdate from hbs_size_property ", con=localdb)['asofdate'][0]

        sql = """SELECT jjdm,cen_lv_rank,asofdate 
        from hbs_size_property where asofdate='{0}' order by cen_lv_rank 
        """.format(latest_asofdate)

        size_equ = pd.read_sql(sql, con=localdb)

        return  size_equ

    @staticmethod
    def pepbroe_equ():

        latest_asofdate = \
            pd.read_sql("select max(asofdate) as asofdate from hbs_holding_property ", con=localdb)[
                'asofdate'][0]

        sql="""
        select jjdm,PE_rank,PB_rank,ROE_rank,股息率_rank,asofdate 
        from hbs_holding_property where asofdate='{0}'
        """.format(latest_asofdate)

        pepbroe_equ=pd.read_sql(sql,con=localdb)
        pepbroe_equ[
            ['PE_rank','PB_rank','ROE_rank','股息率_rank']
        ]=abs(pepbroe_equ[['PE_rank','PB_rank','ROE_rank','股息率_rank']]-0.5)

        pepbroe_equ=pepbroe_equ.sort_values('PE_rank')

        return pepbroe_equ

    @staticmethod
    def nav_equ():

        latest_asofdate = \
            pd.read_sql("select max(asofdate) as asofdate from nav_ret_bias ", con=localdb)[
                'asofdate'][0]

        sql="""
        select jjdm,mean_rank_monthly,std_rank_monthly,mean_rank_weekly,std_rank_weekly,asofdate
        from nav_ret_bias where asofdate='{0}' order by mean_rank_monthly
        """.format(latest_asofdate)

        nav_equ=pd.read_sql(sql,con=localdb)

        return  nav_equ

    def get_equilibrium(self,threshield,show_num=300):

        # method_1
        ind_equ = self.ind_equ()
        style_equ = self.style_equ()
        size_equ = self.size_equ()
        pepbroe_equ = self.pepbroe_equ()
        nav_equ = self.nav_equ()

        #rename columns :
        ind_equ=pd.merge(ind_equ,
                         jjdm_name_map,
                         how='left', on='jjdm').\
            rename(columns={'cen_ind_1':'行业集中度'})

        style_equ=pd.merge(style_equ,
                         jjdm_name_map,
                         how='left', on='jjdm')\
            .rename(columns={'cen_lv_rank': '风格集中度'})

        size_equ=pd.merge(size_equ,
                         jjdm_name_map,
                         how='left', on='jjdm').\
            rename(columns={'cen_lv_rank': '规模集中度'})

        pepbroe_equ=pd.merge(pepbroe_equ,
                         jjdm_name_map,
                         how='left', on='jjdm').rename(columns={'PE_rank': 'pe偏离度',
                                    'PB_rank': 'pb偏离度',
                                    'ROE_rank': 'roe偏离度',
                                    '股息率_rank': '股息率偏离度'})

        nav_equ=pd.merge(nav_equ,
                         jjdm_name_map,
                         how='left', on='jjdm').rename(columns={'mean_rank_monthly': '净值偏离度（月）',
                                'std_rank_monthly': '净值偏离度方差（月）',
                                'mean_rank_weekly': '净值偏离度（周）',
                                'std_rank_weekly': '净值偏离度方差（周）',
                                })

        # method_2
        joint_rank = pd.merge(ind_equ, style_equ, how='inner', on=['jjdm','jjjc'])
        joint_rank = pd.merge(joint_rank, size_equ, how='inner', on=['jjdm','jjjc'])
        joint_rank = pd.merge(joint_rank, pepbroe_equ, how='inner', on=['jjdm','jjjc'])
        joint_rank = pd.merge(joint_rank, nav_equ, how='inner', on=['jjdm','jjjc'])

        col_list = ['行业集中度', '风格集中度',
                    '规模集中度', 'pe偏离度',
                    '净值偏离度（月）']

        joint_rank['平均集中度'] = joint_rank[col_list].mean(axis=1)
        joint_rank = joint_rank.sort_values('平均集中度')

        # method_3
        joint_result = joint_rank[(joint_rank[col_list] <= threshield).prod(axis=1) == 1][['jjdm','jjjc','asofdate'] + col_list]

        joint_rank=joint_rank[['jjdm','jjjc','平均集中度','asofdate']+col_list]


        #format change
        ind_equ['行业集中度'] = ind_equ['行业集中度'].map("{:.2%}".format)
        style_equ['风格集中度'] = style_equ['风格集中度'].map("{:.2%}".format)
        size_equ['规模集中度'] = size_equ['规模集中度'].map("{:.2%}".format)

        for col in ['pe偏离度','pb偏离度','roe偏离度','股息率偏离度']:
            pepbroe_equ[col] = pepbroe_equ[col].map("{:.2%}".format)
        for col in ['净值偏离度（月）', '净值偏离度方差（月）', '净值偏离度（周）', '净值偏离度方差（周）']:
            nav_equ[col] = nav_equ[col].map("{:.2%}".format)
        for col in col_list:
            joint_rank[col]=joint_rank[col].map("{:.2%}".format)
            joint_result[col]=joint_result[col].map("{:.2%}".format)
        joint_rank['平均集中度']=joint_rank['平均集中度'].map("{:.2%}".format)

        return ind_equ[0:show_num],style_equ[0:show_num],\
               size_equ[0:show_num],pepbroe_equ[0:show_num],nav_equ[0:show_num],\
               joint_rank[0:show_num],joint_result[0:show_num]

class Leftside:

    @staticmethod
    def stock_left():

        latest_asofdate=\
            pd.read_sql("select max(asofdate) as asofdate from hbs_stock_trading_property ",
                        con=localdb)['asofdate'][0]

        sql="""
        SELECT jjdm,`左侧概率（出持仓前,半年线）_rank`,`左侧概率（出持仓前,年线）_rank`,asofdate 
        from hbs_stock_trading_property where asofdate='{0}'  """\
            .format(latest_asofdate)
        stock_left=pd.read_sql(sql,con=localdb)

        stock_left['stock_left_rank']=stock_left[['左侧概率（出持仓前,半年线）_rank','左侧概率（出持仓前,年线）_rank']]\
            .max(axis=1)

        stock_left=stock_left.sort_values('stock_left_rank',ascending=False)

        return stock_left

    @staticmethod
    def ind_left():

        latest_asofdate = \
            pd.read_sql("select max(asofdate) as asofdate from hbs_industry_shift_property_new "
                        , con=localdb)['asofdate'][0]
        sql="""
        SELECT jjdm,项目名,Total_rank,asofdate 
        from hbs_industry_shift_property_new where asofdate='{0}' and (项目名='左侧比率' or 项目名='深度左侧比例') 
        """.format(latest_asofdate)
        ind_left=pd.read_sql(sql,con=localdb)

        ind_left=ind_left.groupby('jjdm').max('Total_rank')
        ind_left.reset_index(inplace=True)
        ind_left['asofdate']=latest_asofdate

        ind_left.sort_values('Total_rank',ascending=False,inplace=True)

        ind_left.rename(columns={'Total_rank':'ind_rank'},inplace=True)

        return ind_left

    @staticmethod
    def value_left():

        latest_asofdate = \
            pd.read_sql("select max(asofdate) as asofdate from hbs_shift_property_value "
                        , con=localdb)['asofdate'][0]
        sql="""
        SELECT jjdm,项目名,Total_rank,asofdate 
        from hbs_shift_property_value where asofdate='{0}' and (项目名='左侧比率' or 项目名='深度左侧比例') 
        """.format(latest_asofdate)
        value_left=pd.read_sql(sql,con=localdb)

        value_left=value_left.groupby('jjdm').max('Total_rank')
        value_left.reset_index(inplace=True)
        value_left['asofdate']=latest_asofdate

        value_left.sort_values('Total_rank',ascending=False,inplace=True)
        value_left.rename(columns={'Total_rank': 'value_rank'}, inplace=True)

        return value_left

    @staticmethod
    def size_left():

        latest_asofdate = \
            pd.read_sql("select max(asofdate) as asofdate from hbs_shift_property_size "
                        , con=localdb)['asofdate'][0]
        sql="""
        SELECT jjdm,项目名,Total_rank,asofdate 
        from hbs_shift_property_size where asofdate='{0}' and (项目名='左侧比率' or 项目名='深度左侧比例') 
        """.format(latest_asofdate)
        size_left=pd.read_sql(sql,con=localdb)

        size_left=size_left.groupby('jjdm').max('Total_rank')
        size_left.reset_index(inplace=True)
        size_left['asofdate']=latest_asofdate

        size_left.sort_values('Total_rank',ascending=False,inplace=True)
        size_left.rename(columns={'Total_rank': 'size_rank'}, inplace=True)


        return size_left

    def get_left(self,threshield,show_num=300):

        # method_1
        stock_left = self.stock_left()
        ind_left = self.ind_left()
        value_left = self.value_left()
        size_left = self.size_left()

        #rename columns
        stock_left=stock_left[['jjdm','stock_left_rank','asofdate']]

        stock_left=pd.merge(stock_left,jjdm_name_map,how='left',on='jjdm')\
            .rename(columns={'stock_left_rank':'个股交易左侧率'})
        ind_left = pd.merge(ind_left, jjdm_name_map, how='left', on='jjdm')\
            .rename(columns={'ind_rank':'行业切换左侧率'})
        value_left = pd.merge(value_left, jjdm_name_map, how='left', on='jjdm')\
            .rename(columns={'value_rank':'风格切换左侧率'})
        size_left = pd.merge(size_left, jjdm_name_map, how='left', on='jjdm')\
            .rename(columns={'size_rank':'规模切换左侧率'})


        # method_2
        joint_rank = pd.merge(stock_left, ind_left, how='inner', on=['jjdm','jjjc'])
        joint_rank = pd.merge(joint_rank, value_left, how='inner', on=['jjdm','jjjc'])
        joint_rank = pd.merge(joint_rank, size_left, how='inner', on=['jjdm','jjjc'])
        col_list = ['个股交易左侧率', '行业切换左侧率', '风格切换左侧率', '规模切换左侧率']
        joint_rank['平均左侧率'] = joint_rank[col_list].mean(axis=1)

        # method_3
        joint_restult = joint_rank[(joint_rank[col_list] >= (1 - threshield)).prod(axis=1) == 1][['jjdm','jjjc'] + col_list]

        joint_rank = joint_rank.sort_values('平均左侧率', ascending=False)[['jjdm','jjjc', '平均左侧率', 'asofdate_x']+col_list]


        #re format
        stock_left['个股交易左侧率']=stock_left['个股交易左侧率'].map("{:.2%}".format)
        ind_left['行业切换左侧率'] = ind_left['行业切换左侧率'].map("{:.2%}".format)
        value_left['风格切换左侧率'] = value_left['风格切换左侧率'].map("{:.2%}".format)
        size_left['规模切换左侧率'] = size_left['规模切换左侧率'].map("{:.2%}".format)
        for col in col_list:
            joint_rank[col] = joint_rank[col].map("{:.2%}".format)
            joint_restult[col] = joint_restult[col].map("{:.2%}".format)

        joint_rank['平均左侧率']=joint_rank['平均左侧率'].map("{:.2%}".format)

        return stock_left[0:show_num],ind_left[0:show_num],value_left[0:show_num],size_left[0:show_num],joint_rank[0:show_num],joint_restult[0:show_num]

class Size:
    @staticmethod
    def size_property(fre):
        latest_asofdate = pd.read_sql("select max(asofdate) as asofdate from hbs_size_property".format(fre), con=localdb)['asofdate'][0]
        sql = "select jjdm, shift_lv, cen_lv, shift_lv_rank, cen_lv_rank,大盘, 中盘, 小盘 ,大盘_rank, 中盘_rank, 小盘_rank, asofdate from hbs_size_property where asofdate='{0}'".format( latest_asofdate)
        size_property = pd.read_sql(sql, con=localdb)
        return size_property

    def get_size(self, fre, show_num=200, shift_ratio_threshold=0.5, centralization_threshold=0.5):
        size_property = self.size_property(fre)
        size_property.columns = ['jjdm', '换手率', '集中度', '换手率排名', '集中度排名','大盘(绝对值）', '中盘(绝对值）', '小盘(绝对值）' ,'大盘', '中盘', '小盘', 'asofdate']
        size_property = size_property.merge(jjdm_name_map, on=['jjdm'], how='left')

        size = size_property[(size_property['换手率排名'] < shift_ratio_threshold) & (size_property['集中度排名'] > centralization_threshold)]
        big_size = size[(size['大盘'] > size['中盘']) & (size['大盘'] > size['小盘'])].sort_values('大盘', ascending=False)
        medium_size = size[(size['中盘'] > size['大盘']) & (size['中盘'] > size['小盘'])].sort_values('中盘', ascending=False)
        small_size = size[(size['小盘'] > size['大盘']) & (size['小盘'] > size['中盘'])].sort_values('小盘', ascending=False)

        big_size = big_size[['jjdm', 'jjjc', 'asofdate', '大盘']]
        medium_size = medium_size[['jjdm', 'jjjc', 'asofdate', '中盘']]
        small_size = small_size[['jjdm', 'jjjc', 'asofdate', '小盘']]
        big_size['大盘'] = big_size['大盘'].map("{:.2%}".format)
        medium_size['中盘'] = medium_size['中盘'].map("{:.2%}".format)
        small_size['小盘'] = small_size['小盘'].map("{:.2%}".format)
        return big_size[0: show_num], medium_size[0: show_num], small_size[0: show_num]

class Value:
    @staticmethod
    def value_property(fre):
        latest_asofdate = pd.read_sql("select max(asofdate) as asofdate from hbs_style_property ", con=localdb)['asofdate'][0]
        sql = "select jjdm, shift_lv, cen_lv, shift_lv_rank, cen_lv_rank,成长,价值,成长_rank, 价值_rank, asofdate from hbs_style_property where  asofdate='{0}'".format(latest_asofdate)
        value_property = pd.read_sql(sql, con=localdb)
        return value_property

    @staticmethod
    def holding_property():
        latest_asofdate = pd.read_sql("select max(asofdate) as asofdate from hbs_holding_property", con=localdb)['asofdate'][0]
        sql = "select jjdm, PE_rank, PB_rank, PE_REL_rank, PB_REL_rank, ROE_rank, 股息率_rank, asofdate from hbs_holding_property where asofdate='{0}'".format(latest_asofdate)
        holding_property = pd.read_sql(sql, con=localdb)
        return holding_property

    def get_value(self, fre, show_num=200, shift_ratio_threshold=0.5, centralization_threshold=0.5):
        value_property = self.value_property(fre)
        value_property.columns = ['jjdm', '换手率', '集中度', '换手率排名', '集中度排名','成长（绝对值）', '价值（绝对值）' ,'成长', '价值', 'asofdate']
        value_property = value_property.merge(jjdm_name_map, on=['jjdm'], how='left')

        value = value_property[(value_property['换手率排名'] < shift_ratio_threshold) & (value_property['集中度排名'] > centralization_threshold)]
        growth = value[value['成长'] > value['价值']].sort_values('成长', ascending=False)
        value = value[value['价值'] > value['成长']].sort_values('价值', ascending=False)

        growth = growth[['jjdm', 'jjjc', 'asofdate', '成长']]
        value = value[['jjdm', 'jjjc', 'asofdate', '价值']]
        growth['成长'] = growth['成长'].map("{:.2%}".format)
        value['价值'] = value['价值'].map("{:.2%}".format)

        holding_property = self.holding_property()
        holding_property.columns = ['jjdm', 'PE排名', 'PB排名', 'PE相对行业均值排名', 'PB相对行业均值排名', 'ROE排名', '股息率排名', 'asofdate']
        value_holding_property = value.merge(holding_property.drop('asofdate', axis=1), on=['jjdm'], how='left')
        absolute_pe_value = value_holding_property.sort_values('PE排名')
        absolute_pb_value = value_holding_property.sort_values('PB排名')
        relative_pe_value = value_holding_property.sort_values('PE相对行业均值排名')
        relative_pb_value = value_holding_property.sort_values('PB相对行业均值排名')
        dividend_value = value_holding_property.sort_values('股息率排名', ascending=False)
        reverse_value = value_holding_property.sort_values('ROE排名')
        high_quality_value = value_holding_property.sort_values('ROE排名', ascending=False)

        absolute_pe_value = absolute_pe_value[['jjdm', 'jjjc', 'asofdate', '价值', 'PE排名']]
        absolute_pb_value = absolute_pb_value[['jjdm', 'jjjc', 'asofdate', '价值', 'PB排名']]
        relative_pe_value = relative_pe_value[['jjdm', 'jjjc', 'asofdate', '价值', 'PE相对行业均值排名']]
        relative_pb_value = relative_pb_value[['jjdm', 'jjjc', 'asofdate', '价值', 'PB相对行业均值排名']]
        dividend_value = dividend_value[['jjdm', 'jjjc', 'asofdate', '价值', '股息率排名']]
        reverse_value = reverse_value[['jjdm', 'jjjc', 'asofdate', '价值', 'ROE排名']]
        high_quality_value = high_quality_value[['jjdm', 'jjjc', 'asofdate', '价值', 'ROE排名']]

        absolute_pe_value['PE排名'] = absolute_pe_value['PE排名'].map("{:.2%}".format)
        absolute_pb_value['PB排名'] = absolute_pb_value['PB排名'].map("{:.2%}".format)
        relative_pe_value['PE相对行业均值排名'] = relative_pe_value['PE相对行业均值排名'].map("{:.2%}".format)
        relative_pb_value['PB相对行业均值排名'] = relative_pb_value['PB相对行业均值排名'].map("{:.2%}".format)
        dividend_value['股息率排名'] = dividend_value['股息率排名'].map("{:.2%}".format)
        reverse_value['ROE排名'] = reverse_value['ROE排名'].map("{:.2%}".format)
        high_quality_value['ROE排名'] = high_quality_value['ROE排名'].map("{:.2%}".format)
        return growth[0: show_num], value[0: show_num], absolute_pe_value[0: show_num], absolute_pb_value[0: show_num], \
               relative_pe_value[0: show_num], relative_pb_value[0: show_num], dividend_value[0: show_num], \
               reverse_value[0: show_num], high_quality_value[0: show_num]

class Report:


    @staticmethod
    def get_silly_summary(doc,equ_list,equ_name_list,fold,
                          paragraphs_count,pic_paragraphs_list,string_paragraphs_list,title_num):

        from PIL import Image
        from docx.shared import Inches, Pt


        def style_analysis(df,colname):
            style_style=''
            for style in df.index:
                style_style+=str(style)+"占比"+str(round(df.loc[style][colname],2))+"%,"

            return  style_style

        pic_name_list = ['池列表.png',
                         '行业风格分布.png', '1级行业持仓分布_占持仓比例.png',
                         '1级行业持仓分布_占持仓比例排名.png',
                         '2级行业持仓分布_占持仓比例.png',
                         '2级行业持仓分布_占持仓比例排名.png',
                         '3级行业持仓分布_占持仓比例.png',
                         '3级行业持仓分布_占持仓比例排名.png',
                         '风格类型分布.png',
                         '风格专注型分布.png',
                         '成长价值持仓占比.png',
                         '成长价值持仓占比排名.png',
                         '规模风格类型分布.png',
                         '专注型风格分布.png',
                         '个股持仓特征.png',
                         '大中小盘持仓占比.png',
                         '大中小盘持仓占比排名.png',
                         '个股风格类型分布A.png',
                         '个股风格类型分布B.png',
                         '左侧类型分布.png',
                         '新股偏好分布.png',
                         ]

        for i in range(len(equ_list)):
            equ=equ_list[i]
            name=equ_name_list[i]

            doc.add_paragraph("")
            paragraphs_count += 1

            p=doc.add_paragraph("{2}.{3} {0}：对{1}排名前30的基金进行了统计画像，统计结果如下：".format(name, name,title_num,i+1))
            p.style.font.size=Pt(10)
            p.paragraph_format.first_line_indent=p.style.font.size*2
            p.paragraph_format.line_spacing = Pt(20)  # 行距
            string_paragraphs_list.append(paragraphs_count)
            paragraphs_count+=1


            industry_style, class_list, style_type_dis, style_incline_dis, \
            style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
            stock_style_a, stock_style_b, stock_left, stock_new, stock_fin = pla.pool_picturing(equ, returndata=True,save_local_file=True)

            for pic in pic_name_list:
                file_path=fold+"\\"+pic
                try:
                    doc.add_picture(file_path, width=Inches(3), height=Inches(2.8))
                    pic_paragraphs_list.append(paragraphs_count)
                    paragraphs_count += 1
                except Exception as e:
                    pic_temp = Image.open(file_path)
                    pic_temp.save(pic_temp)
                    doc.add_picture(file_path, width=Inches(4.5), height=Inches(2.5))

            ind_style=style_analysis(industry_style,'num')
            style_style=style_analysis(style_type_dis,'风格类型')
            style_weight=style_analysis(style_weight_dis,'成长价值权重分布')
            size_style=style_analysis(style_type_dis2,'规模风格类型')
            size_weight=style_analysis(style_weight_dis2,'成长价值权重分布')
            left_property=style_analysis(stock_left,'左侧类型分布')
            new_stock_property=style_analysis(stock_new,'新股偏好分布')
            stock_hld_property=style_analysis(stock_fin,'个股持仓特征')

            summary="总结来看：从行业上看，{0}池中".format(name)+ind_style+\
                    "一级行业中，{0}占比较高".format(util.list_sql_condition(class_list[0].index[0:3].tolist()))+\
                    "从风格上看，{0}池中".format(name)+style_style+"具体的持仓来看"+style_weight+"。"+\
                    "从规模上看，{0}池中".format(name) + size_style + "具体的持仓来看" + size_weight + "。"+\
                    "从左侧属性上看，{0}池中".format(name) + left_property +"。"+\
                    "从新股属性上看，{0}池中".format(name) + new_stock_property + "。" +\
                    "从个股持仓属性上看，{0}池中".format(name) + stock_hld_property

            doc.add_paragraph("")
            paragraphs_count += 1

            p=doc.add_paragraph(summary)
            p.style.font.size = Pt(10)
            p.paragraph_format.first_line_indent=p.style.font.size*2
            p.paragraph_format.line_spacing = Pt(20)  # 行距
            string_paragraphs_list.append(paragraphs_count)
            paragraphs_count+=1

        return doc,paragraphs_count,pic_paragraphs_list,string_paragraphs_list

    def filter_pool_report(self):

        from docx import Document
        from docx.enum.text import WD_PARAGRAPH_ALIGNMENT

        #save png 2 doc

        equclass=Equilibrium()
        ind_equ, style_equ, size_equ,pepbroe_equ, nav_equ,\
        joint_rank,joint_result=equclass.get_equilibrium(threshield=0.3,show_num=30)
        equ_list=[joint_rank,joint_result]
        equ_name_list=['均衡平均','均衡交集']

        leftclass=Leftside()
        stock_left, ind_left, value_left,size_left,\
        joint_rank, joint_restult=leftclass.get_left(threshield=0.4,show_num=30)
        left_list=[joint_rank, joint_restult]
        left_name_list = ['左侧平均', '左侧交集']

        sizeclass = Size()
        size_list=sizeclass.get_size(fre='M', show_num=30,
                                                               shift_ratio_threshold=0.5,
                                                               centralization_threshold=0.5)
        size_name_list = ['大盘', '中盘', '小盘']

        valueclass = Value()
        value_list=valueclass.get_value(fre='M', show_num=30,
                                        shift_ratio_threshold=0.5, centralization_threshold=0.5)
        value_list=value_list[0:2]
        value_name_list = ['成长', '价值']

        doc = Document()
        fold=r"E:\GitFolder\hbshare\fe\mutual_analysis"

        pic_paragraphs_list=[]
        string_paragraphs_list=[]
        paragraphs_count=0

        doc,paragraphs_count,pic_paragraphs_list,string_paragraphs_list=self.get_silly_summary(doc,equ_list,equ_name_list,fold
                               ,paragraphs_count,pic_paragraphs_list,string_paragraphs_list,title_num=2)
        doc,paragraphs_count,pic_paragraphs_list,string_paragraphs_list=self.get_silly_summary(doc, left_list, left_name_list, fold,
                               paragraphs_count,pic_paragraphs_list,string_paragraphs_list,title_num=3)
        doc,paragraphs_count,pic_paragraphs_list,string_paragraphs_list=self.get_silly_summary(doc, size_list, size_name_list, fold,
                               paragraphs_count,pic_paragraphs_list,string_paragraphs_list,title_num=4)
        doc,paragraphs_count,pic_paragraphs_list,string_paragraphs_list=self.get_silly_summary(doc, value_list, value_name_list, fold,
                               paragraphs_count,pic_paragraphs_list,string_paragraphs_list,title_num=5)

        for j in pic_paragraphs_list:
            doc.paragraphs[j].alignment = WD_PARAGRAPH_ALIGNMENT.LEFT
        for j in string_paragraphs_list:
            doc.paragraphs[j].alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY


        doc.save(r"E:\GitFolder\hbshare\fe\mutual_analysis\筛选报告\筛选池画像报告.docx")


        print('')

class Advance_man:

    @staticmethod
    def advance_man_from3ind():
        target_ind=['电池化学品','锂电池','锂电专用设备','燃料电池','蓄电池及其他电池','光伏电池组件',
                    '硅料硅片','光伏辅材','光伏加工设备','逆变器','风电零部件','风电整机','电网自动化设备',
                    '电工仪器仪表','配电设备','输变电设备','线缆部件及其他','综合电力设备商','工控设备',
                    '机器人','激光设备','其他自动化设备','工程机械器件','工程机械整机','机床工具','金属制品',
                    '磨具磨料','仪器仪表','制冷空调设备','其他通用设备','纺织服装设备','楼宇设备','能源及重型设备',
                    '其他专用设备','轨交设备Ⅲ','半导体材料','半导体设备','分立器件','集成电路封测','集成电路制造',
                    '模拟芯片设计','数字芯片设计','印制电路板','被动元件','LED','光学元件','面板','品牌消费电子',
                    '消费电子零部件及组装','电子化学品Ⅲ','其他电子Ⅲ','电动乘用车','综合乘用车','商用载货车',
                    '商用载客车','底盘与发动机系统','汽车电子电气系统','轮胎轮毂','车身附件及饰件','其他汽车零部件',
                    '航空装备Ⅲ','航天装备Ⅲ','军工电子Ⅲ','地面兵装Ⅲ','航海装备Ⅲ','医疗设备','医疗耗材','体外诊断',
                    '疫苗','血液制品','其他生物制品','化学制剂','原料药','医疗研发外包','诊断服务','医美耗材','冰洗',
                    '空调','（厨卫电器）厨房电器','（黑色家电）彩电','（小家电）清洁小家电','通信网络设备及器件',
                    '通信线缆及配套','通信终端及配件','其他通信设备','安防设备','其他计算机设备','光伏发电','大气治理',
                    '固废治理','（环保设备）环保设备Ⅲ','油气及炼化工程','油田服务']
        sql="SELECT jjdm,sum(`占持仓比例(时序均值)`) as adv_man from jjpic_industry_detail_3 where `行业名称` in({0}) GROUP BY jjdm "\
            .format(util.list_sql_condition(target_ind))
        adv_man=pd.read_sql(sql,con=localdb).sort_values('adv_man',ascending=False)

        return  adv_man
    @staticmethod
    def advance_man_wording():

        key_words=['先进制造','高端制造','智能制造','国产替代','专精特新','新能源','5G','芯片','半导体']
        jjname=pd.DataFrame(data=util.get_mutual_stock_funds('20211231'),columns=['jjdm'])

        sql="select jjdm,jjmc from st_fund.t_st_gm_jjxx "
        jjname=pd.merge(jjname,hbdb.db2df(sql,db='funduser'),how='left',on='jjdm')

        orconditon=[False]*len(jjname)
        for word in key_words:
            orconditon=(orconditon)|(jjname['jjmc'].str.contains(word))

        return  jjname.loc[orconditon]
    @staticmethod
    def advance_man_fromticker(jjdm_list):
        from hbshare.fe.mutual_analysis import holdind_based as hb

        hld=hb.read_hld_fromdb(jjdm_list,start_date='20211231',end_date='20211231')

        ticker_list=pd.read_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\指数成份汇总.xlsx",sheet_name='成份汇总')
        ticker_list['Wind代码']=ticker_list['Wind代码'].astype(str).str.replace("'", "")
        hld=pd.merge(hld,ticker_list,how='left',left_on='zqdm',right_on='Wind代码')
        hld=hld[hld['Wind代码'].notnull()]
        hld=hld.groupby('jjdm').sum()
        hld['zjbl']=hld['zjbl'].rank(method='min')/len(hld)
        hld.rename(columns={'zjbl':'zjbl_ticker'})
        return hld.reset_index()
    @staticmethod
    def advanve_ols(jjdm_list):
        from hbshare.fe.mutual_analysis import  nav_based as nb
        import statsmodels.api as sm

        index_ret=pd.read_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\指数成份汇总.xlsx",
                                  sheet_name='走势').drop('881001.WI',axis=1)
        index_ret[['930820.CSI','931167.CSI','866003.WI']]=\
            index_ret[['930820.CSI','931167.CSI','866003.WI']].pct_change()
        index_ret['Date']=index_ret['Date'].astype(str).str.replace("-","")
        index_ret=index_ret.iloc[2:]
        index_ret.set_index('Date',inplace=True)
        index_ret = index_ret.sort_index().loc['20190311':]

        result=[[],[],[]]
        para_df=pd.DataFrame(data=jjdm_list,columns=['jjdm'])

        for jjdm in jjdm_list:
            print(jjdm)
            jj_ret = nb.get_jj_daily_ret([jjdm])
            jj_ret.index=jj_ret.index.astype(str)
            olsdf = pd.merge(index_ret,jj_ret, how='left', left_index=True, right_index=True).fillna(0)

            laster_3_year = (datetime.datetime.strptime(str(olsdf.index[-1]),
                                                                                          '%Y%m%d') - datetime.timedelta(
                days=365*3)).strftime('%Y%m%d')

            for j in range(3):
                ind=['930820.CSI','931167.CSI','866003.WI'][j]
                para = sm.OLS(olsdf.loc[laster_3_year:][jjdm].values,
                              olsdf.loc[laster_3_year:][
                                  ind].values).fit().params.tolist()
                result[j].append(para[0])

        for j in range(3):
            ind = ['930820.CSI', '931167.CSI', '866003.WI'][j]+'_rank'
            para_df[ind]=result[j]
            para_df[ind]=para_df[ind].rank(method='min')/len(para_df)

        return  para_df

    def get_advance_man(self):

        jjdm_list = util.get_mutual_stock_funds('20211231')

        adv_man_3ind = self.advance_man_from3ind()
        adv_man_ols=self.advanve_ols(jjdm_list)
        adv_man_ticker=self.advance_man_fromticker(jjdm_list)
        adv_man_words=self.advance_man_wording()

        adv_man=pd.merge(adv_man_3ind,adv_man_words
                         ,how='left',on='jjdm')
        adv_man=pd.merge(adv_man,adv_man_ticker
                         ,how='left',on='jjdm')
        adv_man=pd.merge(adv_man,adv_man_ols
                         ,how='left',on='jjdm')

        adv_man.to_excel('先进制造_基于三级行业列表.xlsx')

        print('Done')

if __name__ == '__main__':

    am=Advance_man()
    am.get_advance_man()

    # rp=Report()
    # rp.filter_pool_report()

    #advance_man()


    #
    # equclass=Equilibrium()
    # ind_equ, style_equ,size_equ, \
    # pepbroe_equ, nav_equ,joint_rank, \
    # joint_result=equclass.get_equilibrium(threshield=0.3,show_num=100)
    #
    #
    #
    # plot=functionality.Plot(800,800)
    #
    # plot.plotly_table(ind_equ, 800, 'asdf')
    # pla.pool_picturing(ind_equ)
    #
    # plot.plotly_table(style_equ, 800, 'asdf')
    # pla.pool_picturing(style_equ)
    #
    # plot.plotly_table(size_equ, 800, 'asdf')
    # pla.pool_picturing(size_equ)
    #
    # plot.plotly_table(pepbroe_equ, 800, 'asdf')
    # pla.pool_picturing(pepbroe_equ)
    #
    # plot.plotly_table(nav_equ, 800, 'asdf')
    # pla.pool_picturing(nav_equ)
    #
    # plot.plotly_table(joint_result,800,'asdf')
    # pla.pool_picturing(joint_result)
    #
    # plot.plotly_table(joint_rank, 800, 'asdf')
    # pla.pool_picturing(joint_rank)

    #

    # leftclass=Leftside()
    #
    # stock_left, ind_left, value_left, \
    # size_left, joint_rank, joint_restult=leftclass.get_left(threshield=0.4,show_num=100)
    #
    #
    # plot=functionality.Plot(800,800)
    #
    # plot.plotly_table(stock_left,800,'asdf')
    # pla.pool_picturing(stock_left)
    #
    # plot.plotly_table(ind_left, 800, 'asdf')
    # pla.pool_picturing(ind_left)
    #
    # plot.plotly_table(value_left, 800, 'asdf')
    # pla.pool_picturing(value_left)
    #
    # plot.plotly_table(size_left, 800, 'asdf')
    # pla.pool_picturing(size_left)
    #
    # plot.plotly_table(joint_rank, 800, 'asdf')
    # pla.pool_picturing(joint_rank)

    # plot.plotly_table(joint_restult, 800, 'asdf')
    # pla.pool_picturing(joint_restult)

    # sizeclass = Size()
    # big_size, medium_size, small_size = sizeclass.get_size(fre='M', show_num=100,
    #                                                        shift_ratio_threshold=0.5,
    #                                                        centralization_threshold=0.5)

    #plot = functionality.Plot(800, 800)
    # plot.plotly_table(big_size, 800, 'asdf')
    # pla.pool_picturing(big_size)
    #
    # plot.plotly_table(medium_size, 800, 'asdf')
    # pla.pool_picturing(medium_size)
    #
    # plot.plotly_table(small_size, 800, 'asdf')
    # pla.pool_picturing(small_size)


    # valueclass = Value()
    # growth, value, absolute_pe_value, absolute_pb_value, relative_pe_value, relative_pb_value, \
    # dividend_value, reverse_value, high_quality_value = valueclass.get_value(fre='M', show_num=200, shift_ratio_threshold=0.5, centralization_threshold=0.5)
    #
    # plot = functionality.Plot(800, 800)
    # plot.plotly_table(growth, 800, 'asdf')
    # pla.pool_picturing(growth)
    #
    # plot.plotly_table(value, 800, 'asdf')
    # pla.pool_picturing(value)
    #
    # plot.plotly_table(absolute_pe_value, 800, 'asdf')
    # pla.pool_picturing(absolute_pe_value)
    #
    # plot.plotly_table(absolute_pb_value, 800, 'asdf')
    # pla.pool_picturing(absolute_pb_value)
    #
    # plot.plotly_table(relative_pe_value, 800, 'asdf')
    # pla.pool_picturing(relative_pe_value)
    #
    # plot.plotly_table(relative_pb_value, 800, 'asdf')
    # pla.pool_picturing(relative_pb_value)
    #
    # plot.plotly_table(dividend_value, 800, 'asdf')
    # pla.pool_picturing(dividend_value)
    #
    # plot.plotly_table(reverse_value, 800, 'asdf')
    # pla.pool_picturing(reverse_value)
    #
    # plot.plotly_table(high_quality_value, 800, 'asdf')
    # pla.pool_picturing(high_quality_value)
    #
    # print("Done")


